@@ -20,8 +20,10 @@
 #include "core/fpdfapi/parser/cpdf_string.h"
 #include "core/fpdfdoc/cpdf_filespec.h"
 #include "core/fpdfdoc/cpdf_formcontrol.h"
+#include "core/fxcrt/fx_codepage.h"
 #include "core/fxge/cfx_substfont.h"
 #include "core/fxge/fx_font.h"
+#include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
 
 namespace {
@@ -38,24 +40,24 @@ const struct SupportFieldEncoding {
     {"UHC", 949},
 };
 
-CFX_WideString GetFieldValue(const CPDF_Dictionary& pFieldDict,
-                             const CFX_ByteString& bsEncoding) {
-  const CFX_ByteString csBValue = pFieldDict.GetStringFor("V");
+WideString GetFieldValue(const CPDF_Dictionary& pFieldDict,
+                         const ByteString& bsEncoding) {
+  const ByteString csBValue = pFieldDict.GetStringFor("V");
   for (const auto& encoding : g_fieldEncoding) {
     if (bsEncoding == encoding.m_name)
-      return CFX_WideString::FromCodePage(csBValue.AsStringC(),
-                                          encoding.m_codePage);
+      return WideString::FromCodePage(csBValue.AsStringView(),
+                                      encoding.m_codePage);
   }
-  CFX_ByteString csTemp = csBValue.Left(2);
+  ByteString csTemp = csBValue.Left(2);
   if (csTemp == "\xFF\xFE" || csTemp == "\xFE\xFF")
     return PDF_DecodeText(csBValue);
-  return CFX_WideString::FromLocal(csBValue.AsStringC());
+  return WideString::FromLocal(csBValue.AsStringView());
 }
 
 void AddFont(CPDF_Dictionary*& pFormDict,
              CPDF_Document* pDocument,
              const CPDF_Font* pFont,
-             CFX_ByteString& csNameTag);
+             ByteString* csNameTag);
 
 void InitDict(CPDF_Dictionary*& pFormDict, CPDF_Document* pDocument) {
   if (!pDocument)
@@ -67,30 +69,26 @@ void InitDict(CPDF_Dictionary*& pFormDict, CPDF_Document* pDocument) {
                                                     pFormDict->GetObjNum());
   }
 
-  CFX_ByteString csDA;
+  ByteString csDA;
   if (!pFormDict->KeyExist("DR")) {
-    CFX_ByteString csBaseName;
-    CFX_ByteString csDefault;
+    ByteString csBaseName;
     uint8_t charSet = CPDF_InterForm::GetNativeCharSet();
     CPDF_Font* pFont = CPDF_InterForm::AddStandardFont(pDocument, "Helvetica");
-    if (pFont) {
-      AddFont(pFormDict, pDocument, pFont, csBaseName);
-      csDefault = csBaseName;
-    }
-    if (charSet != FXFONT_ANSI_CHARSET) {
-      CFX_ByteString csFontName =
-          CPDF_InterForm::GetNativeFont(charSet, nullptr);
+    if (pFont)
+      AddFont(pFormDict, pDocument, pFont, &csBaseName);
+
+    if (charSet != FX_CHARSET_ANSI) {
+      ByteString csFontName = CPDF_InterForm::GetNativeFont(charSet, nullptr);
       if (!pFont || csFontName != "Helvetica") {
         pFont = CPDF_InterForm::AddNativeFont(pDocument);
         if (pFont) {
-          csBaseName = "";
-          AddFont(pFormDict, pDocument, pFont, csBaseName);
-          csDefault = csBaseName;
+          csBaseName.clear();
+          AddFont(pFormDict, pDocument, pFont, &csBaseName);
         }
       }
     }
     if (pFont)
-      csDA = "/" + PDF_NameEncode(csDefault) + " 0 Tf";
+      csDA = "/" + PDF_NameEncode(csBaseName) + " 0 Tf";
   }
   if (!csDA.IsEmpty())
     csDA += " ";
@@ -102,8 +100,8 @@ void InitDict(CPDF_Dictionary*& pFormDict, CPDF_Document* pDocument) {
 
 CPDF_Font* GetFont(CPDF_Dictionary* pFormDict,
                    CPDF_Document* pDocument,
-                   CFX_ByteString csNameTag) {
-  CFX_ByteString csAlias = PDF_NameDecode(csNameTag);
+                   const ByteString& csNameTag) {
+  ByteString csAlias = PDF_NameDecode(csNameTag);
   if (!pFormDict || csAlias.IsEmpty())
     return nullptr;
 
@@ -127,7 +125,7 @@ CPDF_Font* GetFont(CPDF_Dictionary* pFormDict,
 CPDF_Font* GetNativeFont(CPDF_Dictionary* pFormDict,
                          CPDF_Document* pDocument,
                          uint8_t charSet,
-                         CFX_ByteString& csNameTag) {
+                         ByteString* csNameTag) {
   if (!pFormDict)
     return nullptr;
 
@@ -140,7 +138,7 @@ CPDF_Font* GetNativeFont(CPDF_Dictionary* pFormDict,
     return nullptr;
 
   for (const auto& it : *pFonts) {
-    const CFX_ByteString& csKey = it.first;
+    const ByteString& csKey = it.first;
     if (!it.second)
       continue;
 
@@ -157,8 +155,8 @@ CPDF_Font* GetNativeFont(CPDF_Dictionary* pFormDict,
     if (!pSubst)
       continue;
 
-    if (pSubst->m_Charset == (int)charSet) {
-      csNameTag = csKey;
+    if (pSubst->m_Charset == static_cast<int>(charSet)) {
+      *csNameTag = csKey;
       return pFind;
     }
   }
@@ -167,7 +165,7 @@ CPDF_Font* GetNativeFont(CPDF_Dictionary* pFormDict,
 
 bool FindFont(CPDF_Dictionary* pFormDict,
               const CPDF_Font* pFont,
-              CFX_ByteString& csNameTag) {
+              ByteString* csNameTag) {
   if (!pFormDict || !pFont)
     return false;
 
@@ -180,7 +178,7 @@ bool FindFont(CPDF_Dictionary* pFormDict,
     return false;
 
   for (const auto& it : *pFonts) {
-    const CFX_ByteString& csKey = it.first;
+    const ByteString& csKey = it.first;
     if (!it.second)
       continue;
     CPDF_Dictionary* pElement = ToDictionary(it.second->GetDirect());
@@ -189,7 +187,7 @@ bool FindFont(CPDF_Dictionary* pFormDict,
     if (pElement->GetStringFor("Type") != "Font")
       continue;
     if (pFont->GetFontDict() == pElement) {
-      csNameTag = csKey;
+      *csNameTag = csKey;
       return true;
     }
   }
@@ -198,9 +196,9 @@ bool FindFont(CPDF_Dictionary* pFormDict,
 
 bool FindFont(CPDF_Dictionary* pFormDict,
               CPDF_Document* pDocument,
-              CFX_ByteString csFontName,
+              ByteString csFontName,
               CPDF_Font*& pFont,
-              CFX_ByteString& csNameTag) {
+              ByteString* csNameTag) {
   if (!pFormDict)
     return false;
 
@@ -211,11 +209,12 @@ bool FindFont(CPDF_Dictionary* pFormDict,
   CPDF_Dictionary* pFonts = pDR->GetDictFor("Font");
   if (!pFonts)
     return false;
+
   if (csFontName.GetLength() > 0)
     csFontName.Remove(' ');
 
   for (const auto& it : *pFonts) {
-    const CFX_ByteString& csKey = it.first;
+    const ByteString& csKey = it.first;
     if (!it.second)
       continue;
 
@@ -228,11 +227,11 @@ bool FindFont(CPDF_Dictionary* pFormDict,
     if (!pFont)
       continue;
 
-    CFX_ByteString csBaseFont;
+    ByteString csBaseFont;
     csBaseFont = pFont->GetBaseFont();
     csBaseFont.Remove(' ');
     if (csBaseFont == csFontName) {
-      csNameTag = csKey;
+      *csNameTag = csKey;
       return true;
     }
   }
@@ -242,15 +241,15 @@ bool FindFont(CPDF_Dictionary* pFormDict,
 void AddFont(CPDF_Dictionary*& pFormDict,
              CPDF_Document* pDocument,
              const CPDF_Font* pFont,
-             CFX_ByteString& csNameTag) {
+             ByteString* csNameTag) {
   if (!pFont)
     return;
   if (!pFormDict)
     InitDict(pFormDict, pDocument);
 
-  CFX_ByteString csTag;
-  if (FindFont(pFormDict, pFont, csTag)) {
-    csNameTag = csTag;
+  ByteString csTag;
+  if (FindFont(pFormDict, pFont, &csTag)) {
+    *csNameTag = csTag;
     return;
   }
   if (!pFormDict)
@@ -264,30 +263,30 @@ void AddFont(CPDF_Dictionary*& pFormDict,
   if (!pFonts)
     pFonts = pDR->SetNewFor<CPDF_Dictionary>("Font");
 
-  if (csNameTag.IsEmpty())
-    csNameTag = pFont->GetBaseFont();
+  if (csNameTag->IsEmpty())
+    *csNameTag = pFont->GetBaseFont();
 
-  csNameTag.Remove(' ');
-  csNameTag = CPDF_InterForm::GenerateNewResourceName(pDR, "Font", 4,
-                                                      csNameTag.c_str());
-  pFonts->SetNewFor<CPDF_Reference>(csNameTag, pDocument,
+  csNameTag->Remove(' ');
+  *csNameTag = CPDF_InterForm::GenerateNewResourceName(pDR, "Font", 4,
+                                                       csNameTag->c_str());
+  pFonts->SetNewFor<CPDF_Reference>(*csNameTag, pDocument,
                                     pFont->GetFontDict()->GetObjNum());
 }
 
 CPDF_Font* AddNativeFont(CPDF_Dictionary*& pFormDict,
                          CPDF_Document* pDocument,
                          uint8_t charSet,
-                         CFX_ByteString& csNameTag) {
+                         ByteString* csNameTag) {
   if (!pFormDict)
     InitDict(pFormDict, pDocument);
 
-  CFX_ByteString csTemp;
-  CPDF_Font* pFont = GetNativeFont(pFormDict, pDocument, charSet, csTemp);
+  ByteString csTemp;
+  CPDF_Font* pFont = GetNativeFont(pFormDict, pDocument, charSet, &csTemp);
   if (pFont) {
-    csNameTag = csTemp;
+    *csNameTag = csTemp;
     return pFont;
   }
-  CFX_ByteString csFontName = CPDF_InterForm::GetNativeFont(charSet, nullptr);
+  ByteString csFontName = CPDF_InterForm::GetNativeFont(charSet, nullptr);
   if (!csFontName.IsEmpty() &&
       FindFont(pFormDict, pDocument, csFontName, pFont, csNameTag)) {
     return pFont;
@@ -301,29 +300,29 @@ CPDF_Font* AddNativeFont(CPDF_Dictionary*& pFormDict,
 
 class CFieldNameExtractor {
  public:
-  explicit CFieldNameExtractor(const CFX_WideString& full_name)
+  explicit CFieldNameExtractor(const WideString& full_name)
       : m_FullName(full_name) {
     m_pCur = m_FullName.c_str();
     m_pEnd = m_pCur + m_FullName.GetLength();
   }
 
-  void GetNext(const wchar_t*& pSubName, FX_STRSIZE& size) {
+  void GetNext(const wchar_t*& pSubName, size_t& size) {
     pSubName = m_pCur;
     while (m_pCur < m_pEnd && m_pCur[0] != L'.')
       m_pCur++;
 
-    size = (FX_STRSIZE)(m_pCur - pSubName);
+    size = static_cast<size_t>(m_pCur - pSubName);
     if (m_pCur < m_pEnd && m_pCur[0] == L'.')
       m_pCur++;
   }
 
  protected:
-  CFX_WideString m_FullName;
+  WideString m_FullName;
   const wchar_t* m_pCur;
   const wchar_t* m_pEnd;
 };
 
-#if _FXM_PLATFORM_ == _FXM_PLATFORM_WINDOWS_
+#if _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
 typedef struct {
   bool bFind;
   LOGFONTA lf;
@@ -369,15 +368,15 @@ bool RetrieveSpecificFont(uint8_t charSet,
   }
   return RetrieveSpecificFont(lf);
 }
-#endif  // _FXM_PLATFORM_ == _FXM_PLATFORM_WINDOWS_
+#endif  // _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
 
-int CompareFieldName(const CFX_WideString& name1, const CFX_WideString& name2) {
+int CompareFieldName(const WideString& name1, const WideString& name2) {
   const wchar_t* ptr1 = name1.c_str();
   const wchar_t* ptr2 = name2.c_str();
   if (name1.GetLength() == name2.GetLength())
     return name1 == name2 ? 1 : 0;
 
-  int i = 0;
+  size_t i = 0;
   while (ptr1[i] == ptr2[i])
     i++;
   if (i == name1.GetLength())
@@ -393,8 +392,9 @@ class CFieldTree {
  public:
   class Node {
    public:
-    Node() : m_pField(nullptr) {}
-    explicit Node(const CFX_WideString& short_name) : m_ShortName(short_name) {}
+    Node() : m_pField(nullptr), m_level(0) {}
+    Node(const WideString& short_name, int level)
+        : m_ShortName(short_name), m_level(level) {}
     ~Node() {}
 
     void AddChildNode(std::unique_ptr<Node> pNode) {
@@ -411,7 +411,7 @@ class CFieldTree {
       return GetFieldInternal(&nFieldsToGo);
     }
 
-    size_t CountFields() const { return CountFieldsInternal(0); }
+    size_t CountFields() const { return CountFieldsInternal(); }
 
     void SetField(std::unique_ptr<CPDF_FormField> pField) {
       m_pField = std::move(pField);
@@ -419,7 +419,9 @@ class CFieldTree {
 
     CPDF_FormField* GetField() const { return m_pField.get(); }
 
-    const CFX_WideString& GetShortName() const { return m_ShortName; }
+    const WideString& GetShortName() const { return m_ShortName; }
+
+    int GetLevel() const { return m_level; }
 
    private:
     CPDF_FormField* GetFieldInternal(size_t* pFieldsToGo) {
@@ -437,35 +439,33 @@ class CFieldTree {
       return nullptr;
     }
 
-    size_t CountFieldsInternal(int nLevel) const {
-      if (nLevel > nMaxRecursion)
-        return 0;
-
+    size_t CountFieldsInternal() const {
       size_t count = 0;
       if (m_pField)
         ++count;
 
       for (size_t i = 0; i < GetChildrenCount(); ++i)
-        count += GetChildAt(i)->CountFieldsInternal(nLevel + 1);
+        count += GetChildAt(i)->CountFieldsInternal();
       return count;
     }
 
     std::vector<std::unique_ptr<Node>> m_Children;
-    CFX_WideString m_ShortName;
+    WideString m_ShortName;
     std::unique_ptr<CPDF_FormField> m_pField;
+    const int m_level;
   };
 
   CFieldTree();
   ~CFieldTree();
 
-  bool SetField(const CFX_WideString& full_name,
+  bool SetField(const WideString& full_name,
                 std::unique_ptr<CPDF_FormField> pField);
-  CPDF_FormField* GetField(const CFX_WideString& full_name);
+  CPDF_FormField* GetField(const WideString& full_name);
 
-  Node* FindNode(const CFX_WideString& full_name);
-  Node* AddChild(Node* pParent, const CFX_WideString& short_name);
+  Node* FindNode(const WideString& full_name);
+  Node* AddChild(Node* pParent, const WideString& short_name);
 
-  Node* Lookup(Node* pParent, const CFX_WideString& short_name);
+  Node* Lookup(Node* pParent, const WideString& short_name);
 
   Node m_Root;
 };
@@ -475,18 +475,22 @@ CFieldTree::CFieldTree() {}
 CFieldTree::~CFieldTree() {}
 
 CFieldTree::Node* CFieldTree::AddChild(Node* pParent,
-                                       const CFX_WideString& short_name) {
+                                       const WideString& short_name) {
   if (!pParent)
     return nullptr;
 
-  auto pNew = pdfium::MakeUnique<Node>(short_name);
+  int level = pParent->GetLevel() + 1;
+  if (level > nMaxRecursion)
+    return nullptr;
+
+  auto pNew = pdfium::MakeUnique<Node>(short_name, pParent->GetLevel() + 1);
   Node* pChild = pNew.get();
   pParent->AddChildNode(std::move(pNew));
   return pChild;
 }
 
 CFieldTree::Node* CFieldTree::Lookup(Node* pParent,
-                                     const CFX_WideString& short_name) {
+                                     const WideString& short_name) {
   if (!pParent)
     return nullptr;
 
@@ -498,23 +502,25 @@ CFieldTree::Node* CFieldTree::Lookup(Node* pParent,
   return nullptr;
 }
 
-bool CFieldTree::SetField(const CFX_WideString& full_name,
+bool CFieldTree::SetField(const WideString& full_name,
                           std::unique_ptr<CPDF_FormField> pField) {
   if (full_name.IsEmpty())
     return false;
 
   CFieldNameExtractor name_extractor(full_name);
   const wchar_t* pName;
-  FX_STRSIZE nLength;
+  size_t nLength;
   name_extractor.GetNext(pName, nLength);
   Node* pNode = &m_Root;
   Node* pLast = nullptr;
   while (nLength > 0) {
     pLast = pNode;
-    CFX_WideString name = CFX_WideString(pName, nLength);
+    WideString name = WideString(pName, nLength);
     pNode = Lookup(pLast, name);
     if (!pNode)
       pNode = AddChild(pLast, name);
+    if (!pNode)
+      return false;
 
     name_extractor.GetNext(pName, nLength);
   }
@@ -525,38 +531,38 @@ bool CFieldTree::SetField(const CFX_WideString& full_name,
   return true;
 }
 
-CPDF_FormField* CFieldTree::GetField(const CFX_WideString& full_name) {
+CPDF_FormField* CFieldTree::GetField(const WideString& full_name) {
   if (full_name.IsEmpty())
     return nullptr;
 
   CFieldNameExtractor name_extractor(full_name);
   const wchar_t* pName;
-  FX_STRSIZE nLength;
+  size_t nLength;
   name_extractor.GetNext(pName, nLength);
   Node* pNode = &m_Root;
   Node* pLast = nullptr;
   while (nLength > 0 && pNode) {
     pLast = pNode;
-    CFX_WideString name = CFX_WideString(pName, nLength);
+    WideString name = WideString(pName, nLength);
     pNode = Lookup(pLast, name);
     name_extractor.GetNext(pName, nLength);
   }
   return pNode ? pNode->GetField() : nullptr;
 }
 
-CFieldTree::Node* CFieldTree::FindNode(const CFX_WideString& full_name) {
+CFieldTree::Node* CFieldTree::FindNode(const WideString& full_name) {
   if (full_name.IsEmpty())
     return nullptr;
 
   CFieldNameExtractor name_extractor(full_name);
   const wchar_t* pName;
-  FX_STRSIZE nLength;
+  size_t nLength;
   name_extractor.GetNext(pName, nLength);
   Node* pNode = &m_Root;
   Node* pLast = nullptr;
   while (nLength > 0 && pNode) {
     pLast = pNode;
-    CFX_WideString name = CFX_WideString(pName, nLength);
+    WideString name = WideString(pName, nLength);
     pNode = Lookup(pLast, name);
     name_extractor.GetNext(pName, nLength);
   }
@@ -565,64 +571,64 @@ CFieldTree::Node* CFieldTree::FindNode(const CFX_WideString& full_name) {
 
 CPDF_Font* AddNativeInterFormFont(CPDF_Dictionary*& pFormDict,
                                   CPDF_Document* pDocument,
-                                  CFX_ByteString& csNameTag) {
+                                  ByteString* csNameTag) {
   uint8_t charSet = CPDF_InterForm::GetNativeCharSet();
   return AddNativeFont(pFormDict, pDocument, charSet, csNameTag);
 }
 
 // static
 uint8_t CPDF_InterForm::GetNativeCharSet() {
-#if _FXM_PLATFORM_ == _FXM_PLATFORM_WINDOWS_
-  uint8_t charSet = FXFONT_ANSI_CHARSET;
+#if _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
+  uint8_t charSet = FX_CHARSET_ANSI;
   UINT iCodePage = ::GetACP();
   switch (iCodePage) {
-    case 932:
-      charSet = FXFONT_SHIFTJIS_CHARSET;
+    case FX_CODEPAGE_ShiftJIS:
+      charSet = FX_CHARSET_ShiftJIS;
       break;
-    case 936:
-      charSet = FXFONT_GB2312_CHARSET;
+    case FX_CODEPAGE_ChineseSimplified:
+      charSet = FX_CHARSET_ChineseSimplified;
       break;
-    case 950:
-      charSet = FXFONT_CHINESEBIG5_CHARSET;
+    case FX_CODEPAGE_ChineseTraditional:
+      charSet = FX_CHARSET_ChineseTraditional;
       break;
-    case 1252:
-      charSet = FXFONT_ANSI_CHARSET;
+    case FX_CODEPAGE_MSWin_WesternEuropean:
+      charSet = FX_CHARSET_ANSI;
       break;
-    case 874:
-      charSet = FXFONT_THAI_CHARSET;
+    case FX_CODEPAGE_MSDOS_Thai:
+      charSet = FX_CHARSET_Thai;
       break;
-    case 949:
-      charSet = FXFONT_HANGUL_CHARSET;
+    case FX_CODEPAGE_Hangul:
+      charSet = FX_CHARSET_Hangul;
       break;
-    case 1200:
-      charSet = FXFONT_ANSI_CHARSET;
+    case FX_CODEPAGE_UTF16LE:
+      charSet = FX_CHARSET_ANSI;
       break;
-    case 1250:
-      charSet = FXFONT_EASTEUROPE_CHARSET;
+    case FX_CODEPAGE_MSWin_EasternEuropean:
+      charSet = FX_CHARSET_MSWin_EasternEuropean;
       break;
-    case 1251:
-      charSet = FXFONT_RUSSIAN_CHARSET;
+    case FX_CODEPAGE_MSWin_Cyrillic:
+      charSet = FX_CHARSET_MSWin_Cyrillic;
       break;
-    case 1253:
-      charSet = FXFONT_GREEK_CHARSET;
+    case FX_CODEPAGE_MSWin_Greek:
+      charSet = FX_CHARSET_MSWin_Greek;
       break;
-    case 1254:
-      charSet = FXFONT_TURKISH_CHARSET;
+    case FX_CODEPAGE_MSWin_Turkish:
+      charSet = FX_CHARSET_MSWin_Turkish;
       break;
-    case 1255:
-      charSet = FXFONT_HEBREW_CHARSET;
+    case FX_CODEPAGE_MSWin_Hebrew:
+      charSet = FX_CHARSET_MSWin_Hebrew;
       break;
-    case 1256:
-      charSet = FXFONT_ARABIC_CHARSET;
+    case FX_CODEPAGE_MSWin_Arabic:
+      charSet = FX_CHARSET_MSWin_Arabic;
       break;
-    case 1257:
-      charSet = FXFONT_BALTIC_CHARSET;
+    case FX_CODEPAGE_MSWin_Baltic:
+      charSet = FX_CHARSET_MSWin_Baltic;
       break;
-    case 1258:
-      charSet = FXFONT_VIETNAMESE_CHARSET;
+    case FX_CODEPAGE_MSWin_Vietnamese:
+      charSet = FX_CHARSET_MSWin_Vietnamese;
       break;
-    case 1361:
-      charSet = FXFONT_JOHAB_CHARSET;
+    case FX_CODEPAGE_Johab:
+      charSet = FX_CHARSET_Johab;
       break;
   }
   return charSet;
@@ -634,7 +640,7 @@ uint8_t CPDF_InterForm::GetNativeCharSet() {
 CPDF_InterForm::CPDF_InterForm(CPDF_Document* pDocument)
     : m_pDocument(pDocument),
       m_pFormDict(nullptr),
-      m_pFieldTree(new CFieldTree),
+      m_pFieldTree(pdfium::MakeUnique<CFieldTree>()),
       m_pFormNotify(nullptr) {
   CPDF_Dictionary* pRoot = m_pDocument->GetRoot();
   if (!pRoot)
@@ -652,9 +658,7 @@ CPDF_InterForm::CPDF_InterForm(CPDF_Document* pDocument)
     LoadField(pFields->GetDictAt(i), 0);
 }
 
-CPDF_InterForm::~CPDF_InterForm() {
-  m_ControlMap.clear();
-}
+CPDF_InterForm::~CPDF_InterForm() {}
 
 bool CPDF_InterForm::s_bUpdateAP = true;
 
@@ -666,13 +670,13 @@ void CPDF_InterForm::SetUpdateAP(bool bUpdateAP) {
   s_bUpdateAP = bUpdateAP;
 }
 
-CFX_ByteString CPDF_InterForm::GenerateNewResourceName(
+ByteString CPDF_InterForm::GenerateNewResourceName(
     const CPDF_Dictionary* pResDict,
     const char* csType,
     int iMinLen,
     const char* csPrefix) {
-  CFX_ByteString csStr = csPrefix;
-  CFX_ByteString csBType = csType;
+  ByteString csStr = csPrefix;
+  ByteString csBType = csType;
   if (csStr.IsEmpty()) {
     if (csBType == "ExtGState")
       csStr = "GS";
@@ -683,7 +687,7 @@ CFX_ByteString CPDF_InterForm::GenerateNewResourceName(
     else
       csStr = "Res";
   }
-  CFX_ByteString csTmp = csStr;
+  ByteString csTmp = csStr;
   int iCount = csStr.GetLength();
   int m = 0;
   if (iMinLen > 0) {
@@ -705,9 +709,9 @@ CFX_ByteString CPDF_InterForm::GenerateNewResourceName(
     return csTmp;
 
   int num = 0;
-  CFX_ByteString bsNum;
+  ByteString bsNum;
   while (true) {
-    CFX_ByteString csKey = csTmp + bsNum;
+    ByteString csKey = csTmp + bsNum;
     if (!pDict->KeyExist(csKey))
       return csKey;
     if (m < iCount)
@@ -721,7 +725,7 @@ CFX_ByteString CPDF_InterForm::GenerateNewResourceName(
 }
 
 CPDF_Font* CPDF_InterForm::AddStandardFont(CPDF_Document* pDocument,
-                                           CFX_ByteString csFontName) {
+                                           ByteString csFontName) {
   if (!pDocument || csFontName.IsEmpty())
     return nullptr;
 
@@ -732,22 +736,22 @@ CPDF_Font* CPDF_InterForm::AddStandardFont(CPDF_Document* pDocument,
   return pDocument->AddStandardFont(csFontName.c_str(), &encoding);
 }
 
-CFX_ByteString CPDF_InterForm::GetNativeFont(uint8_t charSet, void* pLogFont) {
-  CFX_ByteString csFontName;
-#if _FXM_PLATFORM_ == _FXM_PLATFORM_WINDOWS_
+ByteString CPDF_InterForm::GetNativeFont(uint8_t charSet, void* pLogFont) {
+  ByteString csFontName;
+#if _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
   LOGFONTA lf = {};
-  if (charSet == FXFONT_ANSI_CHARSET) {
+  if (charSet == FX_CHARSET_ANSI) {
     csFontName = "Helvetica";
     return csFontName;
   }
   bool bRet = false;
-  if (charSet == FXFONT_SHIFTJIS_CHARSET) {
+  if (charSet == FX_CHARSET_ShiftJIS) {
     bRet = RetrieveSpecificFont(charSet, DEFAULT_PITCH | FF_DONTCARE,
                                 "MS Mincho", lf);
-  } else if (charSet == FXFONT_GB2312_CHARSET) {
+  } else if (charSet == FX_CHARSET_ChineseSimplified) {
     bRet = RetrieveSpecificFont(charSet, DEFAULT_PITCH | FF_DONTCARE, "SimSun",
                                 lf);
-  } else if (charSet == FXFONT_CHINESEBIG5_CHARSET) {
+  } else if (charSet == FX_CHARSET_ChineseTraditional) {
     bRet = RetrieveSpecificFont(charSet, DEFAULT_PITCH | FF_DONTCARE, "MingLiU",
                                 lf);
   }
@@ -779,9 +783,9 @@ CPDF_Font* CPDF_InterForm::AddNativeFont(uint8_t charSet,
   if (!pDocument)
     return nullptr;
 
-#if _FXM_PLATFORM_ == _FXM_PLATFORM_WINDOWS_
+#if _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
   LOGFONTA lf;
-  CFX_ByteString csFontName = GetNativeFont(charSet, &lf);
+  ByteString csFontName = GetNativeFont(charSet, &lf);
   if (!csFontName.IsEmpty()) {
     if (csFontName == "Helvetica")
       return AddStandardFont(pDocument, csFontName);
@@ -796,7 +800,7 @@ CPDF_Font* CPDF_InterForm::AddNativeFont(CPDF_Document* pDocument) {
 }
 
 bool CPDF_InterForm::ValidateFieldName(
-    CFX_WideString& csNewFieldName,
+    WideString& csNewFieldName,
     int iType,
     const CPDF_FormField* pExcludedField,
     const CPDF_FormControl* pExcludedControl) const {
@@ -805,7 +809,7 @@ bool CPDF_InterForm::ValidateFieldName(
 
   int iPos = 0;
   int iLength = csNewFieldName.GetLength();
-  CFX_WideString csSub;
+  WideString csSub;
   while (true) {
     while (iPos < iLength &&
            (csNewFieldName[iPos] == L'.' || csNewFieldName[iPos] == L' ')) {
@@ -830,7 +834,7 @@ bool CPDF_InterForm::ValidateFieldName(
         if (!pExcludedControl || pField->CountControls() < 2)
           continue;
       }
-      CFX_WideString csFullName = pField->GetFullName();
+      WideString csFullName = pField->GetFullName();
       int iRet = CompareFieldName(csSub, csFullName);
       if (iRet == 1) {
         if (pField->GetFieldType() != iType)
@@ -853,7 +857,7 @@ bool CPDF_InterForm::ValidateFieldName(
   return true;
 }
 
-size_t CPDF_InterForm::CountFields(const CFX_WideString& csFieldName) const {
+size_t CPDF_InterForm::CountFields(const WideString& csFieldName) const {
   if (csFieldName.IsEmpty())
     return m_pFieldTree->m_Root.CountFields();
 
@@ -861,9 +865,8 @@ size_t CPDF_InterForm::CountFields(const CFX_WideString& csFieldName) const {
   return pNode ? pNode->CountFields() : 0;
 }
 
-CPDF_FormField* CPDF_InterForm::GetField(
-    uint32_t index,
-    const CFX_WideString& csFieldName) const {
+CPDF_FormField* CPDF_InterForm::GetField(uint32_t index,
+                                         const WideString& csFieldName) const {
   if (csFieldName.IsEmpty())
     return m_pFieldTree->m_Root.GetFieldAtIndex(index);
 
@@ -876,7 +879,7 @@ CPDF_FormField* CPDF_InterForm::GetFieldByDict(
   if (!pFieldDict)
     return nullptr;
 
-  CFX_WideString csWName = FPDF_GetFullName(pFieldDict);
+  WideString csWName = FPDF_GetFullName(pFieldDict);
   return m_pFieldTree->GetField(csWName);
 }
 
@@ -949,14 +952,14 @@ int CPDF_InterForm::FindFieldInCalculationOrder(const CPDF_FormField* pField) {
 
   for (size_t i = 0; i < pArray->GetCount(); i++) {
     CPDF_Object* pElement = pArray->GetDirectObjectAt(i);
-    if (pElement == pField->m_pDict)
+    if (pElement == pField->GetDict())
       return i;
   }
   return -1;
 }
 
-CPDF_Font* CPDF_InterForm::GetFormFont(CFX_ByteString csNameTag) {
-  return GetFont(m_pFormDict, m_pDocument, csNameTag);
+CPDF_Font* CPDF_InterForm::GetFormFont(ByteString csNameTag) const {
+  return GetFont(m_pFormDict.Get(), m_pDocument.Get(), csNameTag);
 }
 
 CPDF_DefaultAppearance CPDF_InterForm::GetDefaultAppearance() const {
@@ -1041,7 +1044,7 @@ bool CPDF_InterForm::HasXFAForm() const {
 }
 
 void CPDF_InterForm::FixPageFields(const CPDF_Page* pPage) {
-  CPDF_Dictionary* pPageDict = pPage->m_pFormDict;
+  CPDF_Dictionary* pPageDict = pPage->m_pFormDict.Get();
   if (!pPageDict)
     return;
 
@@ -1065,7 +1068,7 @@ void CPDF_InterForm::AddTerminalField(CPDF_Dictionary* pFieldDict) {
   }
 
   CPDF_Dictionary* pDict = pFieldDict;
-  CFX_WideString csWName = FPDF_GetFullName(pFieldDict);
+  WideString csWName = FPDF_GetFullName(pFieldDict);
   if (csWName.IsEmpty())
     return;
 
@@ -1134,7 +1137,7 @@ CPDF_FormControl* CPDF_InterForm::AddControl(CPDF_FormField* pField,
   auto pNew = pdfium::MakeUnique<CPDF_FormControl>(pField, pWidgetDict);
   CPDF_FormControl* pControl = pNew.get();
   m_ControlMap[pWidgetDict] = std::move(pNew);
-  pField->m_ControlList.push_back(pControl);
+  pField->AddFormControl(pControl);
   return pControl;
 }
 
@@ -1161,7 +1164,7 @@ bool CPDF_InterForm::CheckRequiredFields(
     if (fields)
       bFind = pdfium::ContainsValue(*fields, pField);
     if (bIncludeOrExclude == bFind) {
-      CPDF_Dictionary* pFieldDict = pField->m_pDict;
+      const CPDF_Dictionary* pFieldDict = pField->GetDict();
       if ((dwFlags & FORMFLAG_REQUIRED) != 0 &&
           pFieldDict->GetStringFor("V").IsEmpty()) {
         return false;
@@ -1172,7 +1175,7 @@ bool CPDF_InterForm::CheckRequiredFields(
 }
 
 std::unique_ptr<CFDF_Document> CPDF_InterForm::ExportToFDF(
-    const CFX_WideStringC& pdf_path,
+    const WideString& pdf_path,
     bool bSimpleFileSpec) const {
   std::vector<CPDF_FormField*> fields;
   size_t nCount = m_pFieldTree->m_Root.CountFields();
@@ -1182,7 +1185,7 @@ std::unique_ptr<CFDF_Document> CPDF_InterForm::ExportToFDF(
 }
 
 std::unique_ptr<CFDF_Document> CPDF_InterForm::ExportToFDF(
-    const CFX_WideStringC& pdf_path,
+    const WideString& pdf_path,
     const std::vector<CPDF_FormField*>& fields,
     bool bIncludeOrExclude,
     bool bSimpleFileSpec) const {
@@ -1193,15 +1196,18 @@ std::unique_ptr<CFDF_Document> CPDF_InterForm::ExportToFDF(
   CPDF_Dictionary* pMainDict = pDoc->GetRoot()->GetDictFor("FDF");
   if (!pdf_path.IsEmpty()) {
     if (bSimpleFileSpec) {
-      CFX_WideString wsFilePath = CPDF_FileSpec::EncodeFileName(pdf_path);
+      WideString wsFilePath = CPDF_FileSpec::EncodeFileName(pdf_path);
       pMainDict->SetNewFor<CPDF_String>(
-          "F", CFX_ByteString::FromUnicode(wsFilePath), false);
+          "F", ByteString::FromUnicode(wsFilePath), false);
       pMainDict->SetNewFor<CPDF_String>("UF", PDF_EncodeText(wsFilePath),
                                         false);
     } else {
-      CPDF_FileSpec filespec(pDoc->GetByteStringPool());
+      auto pNewDict =
+          pdfium::MakeUnique<CPDF_Dictionary>(pDoc->GetByteStringPool());
+      pNewDict->SetNewFor<CPDF_Name>("Type", "Filespec");
+      CPDF_FileSpec filespec(pNewDict.get());
       filespec.SetFileName(pdf_path);
-      pMainDict->SetFor("F", pdfium::WrapUnique(filespec.GetObj()));
+      pMainDict->SetFor("F", std::move(pNewDict));
     }
   }
 
@@ -1217,24 +1223,26 @@ std::unique_ptr<CFDF_Document> CPDF_InterForm::ExportToFDF(
       continue;
 
     if (bIncludeOrExclude == pdfium::ContainsValue(fields, pField)) {
-      if ((dwFlags & 0x02) != 0 && pField->m_pDict->GetStringFor("V").IsEmpty())
+      if ((dwFlags & 0x02) != 0 &&
+          pField->GetDict()->GetStringFor("V").IsEmpty()) {
         continue;
+      }
 
-      CFX_WideString fullname = FPDF_GetFullName(pField->GetFieldDict());
+      WideString fullname = FPDF_GetFullName(pField->GetFieldDict());
       auto pFieldDict =
           pdfium::MakeUnique<CPDF_Dictionary>(pDoc->GetByteStringPool());
       pFieldDict->SetNewFor<CPDF_String>("T", fullname);
       if (pField->GetType() == CPDF_FormField::CheckBox ||
           pField->GetType() == CPDF_FormField::RadioButton) {
-        CFX_WideString csExport = pField->GetCheckValue(false);
-        CFX_ByteString csBExport = PDF_EncodeText(csExport);
-        CPDF_Object* pOpt = FPDF_GetFieldAttr(pField->m_pDict, "Opt");
+        WideString csExport = pField->GetCheckValue(false);
+        ByteString csBExport = PDF_EncodeText(csExport);
+        CPDF_Object* pOpt = FPDF_GetFieldAttr(pField->GetDict(), "Opt");
         if (pOpt)
           pFieldDict->SetNewFor<CPDF_String>("V", csBExport, false);
         else
           pFieldDict->SetNewFor<CPDF_Name>("V", csBExport);
       } else {
-        CPDF_Object* pV = FPDF_GetFieldAttr(pField->m_pDict, "V");
+        CPDF_Object* pV = FPDF_GetFieldAttr(pField->GetDict(), "V");
         if (pV)
           pFieldDict->SetFor("V", pV->CloneDirectObject());
       }
@@ -1245,10 +1253,10 @@ std::unique_ptr<CFDF_Document> CPDF_InterForm::ExportToFDF(
 }
 
 void CPDF_InterForm::FDF_ImportField(CPDF_Dictionary* pFieldDict,
-                                     const CFX_WideString& parent_name,
+                                     const WideString& parent_name,
                                      bool bNotify,
                                      int nLevel) {
-  CFX_WideString name;
+  WideString name;
   if (!parent_name.IsEmpty())
     name = parent_name + L".";
 
@@ -1271,7 +1279,7 @@ void CPDF_InterForm::FDF_ImportField(CPDF_Dictionary* pFieldDict,
   if (!pField)
     return;
 
-  CFX_WideString csWValue = GetFieldValue(*pFieldDict, m_bsEncoding);
+  WideString csWValue = GetFieldValue(*pFieldDict, m_bsEncoding);
   int iType = pField->GetFieldType();
   if (bNotify && m_pFormNotify) {
     if (iType == FIELDTYPE_LISTBOX) {
@@ -1286,8 +1294,7 @@ void CPDF_InterForm::FDF_ImportField(CPDF_Dictionary* pFieldDict,
   CPDF_FormField::Type eType = pField->GetType();
   if ((eType == CPDF_FormField::ListBox || eType == CPDF_FormField::ComboBox) &&
       pFieldDict->KeyExist("Opt")) {
-    pField->m_pDict->SetFor(
-        "Opt", pFieldDict->GetDirectObjectFor("Opt")->CloneDirectObject());
+    pField->SetOpt(pFieldDict->GetDirectObjectFor("Opt")->CloneDirectObject());
   }
 
   if (bNotify && m_pFormNotify) {
