@@ -8,8 +8,10 @@
 #include <memory>
 #include <vector>
 
+#include "core/fxcodec/codec/ccodec_faxmodule.h"
+#include "core/fxcodec/codec/ccodec_scanlinedecoder.h"
 #include "core/fxcodec/codec/codec_int.h"
-#include "core/fxcodec/fx_codec.h"
+#include "core/fxcrt/cfx_binarybuf.h"
 #include "core/fxcrt/fx_memory.h"
 #include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
@@ -605,7 +607,7 @@ std::unique_ptr<CCodec_ScanlineDecoder> CCodec_FaxModule::CreateDecoder(
       EncodedByteAlign, BlackIs1);
 }
 
-#if _FX_OS_ == _FX_WIN32_DESKTOP_ || _FX_OS_ == _FX_WIN64_DESKTOP_
+#if _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
 namespace {
 const uint8_t BlackRunTerminator[128] = {
     0x37, 10, 0x02, 3,  0x03, 2,  0x02, 2,  0x03, 3,  0x03, 4,  0x02, 4,
@@ -799,4 +801,4 @@ void CCodec_FaxModule::FaxEncode(
   encoder.Encode(dest_buf, dest_size);
 }
 
-#endif
+#endif  // _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
