@@ -7,6 +7,7 @@
 #include "core/fxcodec/jbig2/JBig2_Context.h"
 
 #include <algorithm>
+#include <limits>
 #include <list>
 #include <utility>
 #include <vector>
@@ -22,6 +23,7 @@
 #include "core/fxcodec/jbig2/JBig2_PddProc.h"
 #include "core/fxcodec/jbig2/JBig2_SddProc.h"
 #include "core/fxcodec/jbig2/JBig2_TrdProc.h"
+#include "core/fxcrt/ifx_pauseindicator.h"
 #include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
 
@@ -46,31 +48,29 @@ size_t GetRefAggContextSize(bool val) {
 // difference for typical JBIG2 documents.
 static const int kSymbolDictCacheMaxSize = 2;
 
-CJBig2_Context::CJBig2_Context(
-    const CFX_RetainPtr<CPDF_StreamAcc>& pGlobalStream,
-    const CFX_RetainPtr<CPDF_StreamAcc>& pSrcStream,
-    std::list<CJBig2_CachePair>* pSymbolDictCache,
-    IFX_Pause* pPause,
-    bool bIsGlobal)
+CJBig2_Context::CJBig2_Context(const RetainPtr<CPDF_StreamAcc>& pGlobalStream,
+                               const RetainPtr<CPDF_StreamAcc>& pSrcStream,
+                               std::list<CJBig2_CachePair>* pSymbolDictCache,
+                               bool bIsGlobal)
     : m_nSegmentDecoded(0),
       m_bInPage(false),
       m_bBufSpecified(false),
       m_PauseStep(10),
-      m_pPause(pPause),
       m_ProcessingStatus(FXCODEC_STATUS_FRAME_READY),
       m_dwOffset(0),
       m_pSymbolDictCache(pSymbolDictCache),
       m_bIsGlobal(bIsGlobal) {
-  if (pGlobalStream && (pGlobalStream->GetSize() > 0)) {
+  if (pGlobalStream && pGlobalStream->GetSize() > 0) {
     m_pGlobalContext = pdfium::MakeUnique<CJBig2_Context>(
-        nullptr, pGlobalStream, pSymbolDictCache, pPause, true);
+        nullptr, pGlobalStream, pSymbolDictCache, true);
   }
   m_pStream = pdfium::MakeUnique<CJBig2_BitStream>(pSrcStream);
 }
 
 CJBig2_Context::~CJBig2_Context() {}
 
-int32_t CJBig2_Context::decode_SquentialOrgnazation(IFX_Pause* pPause) {
+int32_t CJBig2_Context::decode_SquentialOrgnazation(
+    IFX_PauseIndicator* pPause) {
   int32_t nRet;
   if (m_pStream->getByteLeft() <= 0)
     return JBIG2_END_OF_FILE;
@@ -116,11 +116,12 @@ int32_t CJBig2_Context::decode_SquentialOrgnazation(IFX_Pause* pPause) {
   return JBIG2_SUCCESS;
 }
 
-int32_t CJBig2_Context::decode_EmbedOrgnazation(IFX_Pause* pPause) {
+int32_t CJBig2_Context::decode_EmbedOrgnazation(IFX_PauseIndicator* pPause) {
   return decode_SquentialOrgnazation(pPause);
 }
 
-int32_t CJBig2_Context::decode_RandomOrgnazation_FirstPage(IFX_Pause* pPause) {
+int32_t CJBig2_Context::decode_RandomOrgnazation_FirstPage(
+    IFX_PauseIndicator* pPause) {
   int32_t nRet;
   while (m_pStream->getByteLeft() > JBIG2_MIN_SEGMENT_SIZE) {
     auto pSegment = pdfium::MakeUnique<CJBig2_Segment>();
@@ -131,7 +132,7 @@ int32_t CJBig2_Context::decode_RandomOrgnazation_FirstPage(IFX_Pause* pPause) {
       break;
     }
     m_SegmentList.push_back(std::move(pSegment));
-    if (pPause && m_pPause && pPause->NeedToPauseNow()) {
+    if (pPause && pPause->NeedToPauseNow()) {
       m_PauseStep = 3;
       m_ProcessingStatus = FXCODEC_STATUS_DECODE_TOBECONTINUE;
       return JBIG2_SUCCESS;
@@ -141,7 +142,7 @@ int32_t CJBig2_Context::decode_RandomOrgnazation_FirstPage(IFX_Pause* pPause) {
   return decode_RandomOrgnazation(pPause);
 }
 
-int32_t CJBig2_Context::decode_RandomOrgnazation(IFX_Pause* pPause) {
+int32_t CJBig2_Context::decode_RandomOrgnazation(IFX_PauseIndicator* pPause) {
   for (; m_nSegmentDecoded < m_SegmentList.size(); ++m_nSegmentDecoded) {
     int32_t nRet =
         parseSegmentData(m_SegmentList[m_nSegmentDecoded].get(), pPause);
@@ -164,7 +165,7 @@ int32_t CJBig2_Context::getFirstPage(uint8_t* pBuf,
                                      int32_t width,
                                      int32_t height,
                                      int32_t stride,
-                                     IFX_Pause* pPause) {
+                                     IFX_PauseIndicator* pPause) {
   int32_t nRet = 0;
   if (m_pGlobalContext) {
     nRet = m_pGlobalContext->decode_EmbedOrgnazation(pPause);
@@ -184,7 +185,7 @@ int32_t CJBig2_Context::getFirstPage(uint8_t* pBuf,
   return Continue(pPause);
 }
 
-int32_t CJBig2_Context::Continue(IFX_Pause* pPause) {
+int32_t CJBig2_Context::Continue(IFX_PauseIndicator* pPause) {
   m_ProcessingStatus = FXCODEC_STATUS_DECODE_READY;
   int32_t nRet = 0;
   if (m_PauseStep <= 1) {
@@ -236,7 +237,7 @@ CJBig2_Segment* CJBig2_Context::findReferredSegmentByTypeAndIndex(
   int32_t count = 0;
   for (int32_t i = 0; i < pSegment->m_nReferred_to_segment_count; ++i) {
     CJBig2_Segment* pSeg =
-        findSegmentByNumber(pSegment->m_pReferred_to_segment_numbers[i]);
+        findSegmentByNumber(pSegment->m_Referred_to_segment_numbers[i]);
     if (pSeg && pSeg->m_cFlags.s.type == cType) {
       if (count == nIndex)
         return pSeg;
@@ -276,31 +277,31 @@ int32_t CJBig2_Context::parseSegmentHeader(CJBig2_Segment* pSegment) {
       pSegment->m_dwNumber > 65536 ? 4 : pSegment->m_dwNumber > 256 ? 2 : 1;
   uint8_t cPSize = pSegment->m_cFlags.s.page_association_size ? 4 : 1;
   if (pSegment->m_nReferred_to_segment_count) {
-    pSegment->m_pReferred_to_segment_numbers =
-        FX_Alloc(uint32_t, pSegment->m_nReferred_to_segment_count);
+    pSegment->m_Referred_to_segment_numbers.resize(
+        pSegment->m_nReferred_to_segment_count);
     for (int32_t i = 0; i < pSegment->m_nReferred_to_segment_count; ++i) {
       switch (cSSize) {
         case 1:
           if (m_pStream->read1Byte(&cTemp) != 0)
             return JBIG2_ERROR_TOO_SHORT;
 
-          pSegment->m_pReferred_to_segment_numbers[i] = cTemp;
+          pSegment->m_Referred_to_segment_numbers[i] = cTemp;
           break;
         case 2:
           uint16_t wTemp;
           if (m_pStream->readShortInteger(&wTemp) != 0)
             return JBIG2_ERROR_TOO_SHORT;
 
-          pSegment->m_pReferred_to_segment_numbers[i] = wTemp;
+          pSegment->m_Referred_to_segment_numbers[i] = wTemp;
           break;
         case 4:
           if (m_pStream->readInteger(&dwTemp) != 0)
             return JBIG2_ERROR_TOO_SHORT;
 
-          pSegment->m_pReferred_to_segment_numbers[i] = dwTemp;
+          pSegment->m_Referred_to_segment_numbers[i] = dwTemp;
           break;
       }
-      if (pSegment->m_pReferred_to_segment_numbers[i] >= pSegment->m_dwNumber)
+      if (pSegment->m_Referred_to_segment_numbers[i] >= pSegment->m_dwNumber)
         return JBIG2_ERROR_TOO_SHORT;
     }
   }
@@ -323,7 +324,7 @@ int32_t CJBig2_Context::parseSegmentHeader(CJBig2_Segment* pSegment) {
 }
 
 int32_t CJBig2_Context::parseSegmentData(CJBig2_Segment* pSegment,
-                                         IFX_Pause* pPause) {
+                                         IFX_PauseIndicator* pPause) {
   int32_t ret = ProcessingParseSegmentData(pSegment, pPause);
   while (m_ProcessingStatus == FXCODEC_STATUS_DECODE_TOBECONTINUE &&
          m_pStream->getByteLeft() > 0) {
@@ -333,10 +334,10 @@ int32_t CJBig2_Context::parseSegmentData(CJBig2_Segment* pSegment,
 }
 
 int32_t CJBig2_Context::ProcessingParseSegmentData(CJBig2_Segment* pSegment,
-                                                   IFX_Pause* pPause) {
+                                                   IFX_PauseIndicator* pPause) {
   switch (pSegment->m_cFlags.s.type) {
     case 0:
-      return parseSymbolDict(pSegment, pPause);
+      return parseSymbolDict(pSegment);
     case 4:
     case 6:
     case 7:
@@ -419,8 +420,7 @@ int32_t CJBig2_Context::ProcessingParseSegmentData(CJBig2_Segment* pSegment,
   return JBIG2_SUCCESS;
 }
 
-int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
-                                        IFX_Pause* pPause) {
+int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment) {
   uint16_t wFlags;
   if (m_pStream->readShortInteger(&wFlags) != 0)
     return JBIG2_ERROR_TOO_SHORT;
@@ -456,16 +456,16 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
     return JBIG2_ERROR_LIMIT;
   }
   for (int32_t i = 0; i < pSegment->m_nReferred_to_segment_count; ++i) {
-    if (!findSegmentByNumber(pSegment->m_pReferred_to_segment_numbers[i]))
+    if (!findSegmentByNumber(pSegment->m_Referred_to_segment_numbers[i]))
       return JBIG2_ERROR_FATAL;
   }
   CJBig2_Segment* pLRSeg = nullptr;
   pSymbolDictDecoder->SDNUMINSYMS = 0;
   for (int32_t i = 0; i < pSegment->m_nReferred_to_segment_count; ++i) {
     CJBig2_Segment* pSeg =
-        findSegmentByNumber(pSegment->m_pReferred_to_segment_numbers[i]);
+        findSegmentByNumber(pSegment->m_Referred_to_segment_numbers[i]);
     if (pSeg->m_cFlags.s.type == 0) {
-      pSymbolDictDecoder->SDNUMINSYMS += pSeg->m_Result.sd->NumImages();
+      pSymbolDictDecoder->SDNUMINSYMS += pSeg->m_SymbolDict->NumImages();
       pLRSeg = pSeg;
     }
   }
@@ -476,9 +476,9 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
     uint32_t dwTemp = 0;
     for (int32_t i = 0; i < pSegment->m_nReferred_to_segment_count; ++i) {
       CJBig2_Segment* pSeg =
-          findSegmentByNumber(pSegment->m_pReferred_to_segment_numbers[i]);
+          findSegmentByNumber(pSegment->m_Referred_to_segment_numbers[i]);
       if (pSeg->m_cFlags.s.type == 0) {
-        const CJBig2_SymbolDict& dict = *pSeg->m_Result.sd;
+        const CJBig2_SymbolDict& dict = *pSeg->m_SymbolDict.get();
         for (size_t j = 0; j < dict.NumImages(); ++j)
           SDINSYMS.get()[dwTemp + j] = dict.GetImage(j);
         dwTemp += dict.NumImages();
@@ -510,7 +510,7 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pSymbolDictDecoder->SDHUFFDH = pSeg->m_Result.ht;
+      pSymbolDictDecoder->SDHUFFDH = pSeg->m_HuffmanTable.get();
     }
     if (cSDHUFFDW == 0) {
       Table_B2 = pdfium::MakeUnique<CJBig2_HuffmanTable>(
@@ -525,7 +525,7 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pSymbolDictDecoder->SDHUFFDW = pSeg->m_Result.ht;
+      pSymbolDictDecoder->SDHUFFDW = pSeg->m_HuffmanTable.get();
     }
     if (cSDHUFFBMSIZE == 0) {
       Table_B1 = pdfium::MakeUnique<CJBig2_HuffmanTable>(
@@ -536,7 +536,7 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pSymbolDictDecoder->SDHUFFBMSIZE = pSeg->m_Result.ht;
+      pSymbolDictDecoder->SDHUFFBMSIZE = pSeg->m_HuffmanTable.get();
     }
     if (pSymbolDictDecoder->SDREFAGG == 1) {
       if (cSDHUFFAGGINST == 0) {
@@ -550,7 +550,7 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
             findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
         if (!pSeg)
           return JBIG2_ERROR_FATAL;
-        pSymbolDictDecoder->SDHUFFAGGINST = pSeg->m_Result.ht;
+        pSymbolDictDecoder->SDHUFFAGGINST = pSeg->m_HuffmanTable.get();
       }
     }
   }
@@ -565,12 +565,12 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
   std::vector<JBig2ArithCtx> grContext;
   if ((wFlags & 0x0100) && pLRSeg) {
     if (bUseGbContext) {
-      gbContext = pLRSeg->m_Result.sd->GbContext();
+      gbContext = pLRSeg->m_SymbolDict->GbContext();
       if (gbContext.size() != gbContextSize)
         return JBIG2_ERROR_FATAL;
     }
     if (bUseGrContext) {
-      grContext = pLRSeg->m_Result.sd->GrContext();
+      grContext = pLRSeg->m_SymbolDict->GrContext();
       if (grContext.size() != grContextSize)
         return JBIG2_ERROR_FATAL;
     }
@@ -589,8 +589,7 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
     for (auto it = m_pSymbolDictCache->begin(); it != m_pSymbolDictCache->end();
          ++it) {
       if (it->first == key) {
-        std::unique_ptr<CJBig2_SymbolDict> copy(it->second->DeepCopy());
-        pSegment->m_Result.sd = copy.release();
+        pSegment->m_SymbolDict = it->second->DeepCopy();
         m_pSymbolDictCache->push_front(
             CJBig2_CachePair(key, std::move(it->second)));
         m_pSymbolDictCache->erase(it);
@@ -603,23 +602,23 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
     if (bUseGbContext) {
       auto pArithDecoder =
           pdfium::MakeUnique<CJBig2_ArithDecoder>(m_pStream.get());
-      pSegment->m_Result.sd = pSymbolDictDecoder->decode_Arith(
+      pSegment->m_SymbolDict = pSymbolDictDecoder->decode_Arith(
           pArithDecoder.get(), &gbContext, &grContext);
-      if (!pSegment->m_Result.sd)
+      if (!pSegment->m_SymbolDict)
         return JBIG2_ERROR_FATAL;
 
       m_pStream->alignByte();
       m_pStream->offset(2);
     } else {
-      pSegment->m_Result.sd = pSymbolDictDecoder->decode_Huffman(
-          m_pStream.get(), &gbContext, &grContext, pPause);
-      if (!pSegment->m_Result.sd)
+      pSegment->m_SymbolDict = pSymbolDictDecoder->decode_Huffman(
+          m_pStream.get(), &gbContext, &grContext);
+      if (!pSegment->m_SymbolDict)
         return JBIG2_ERROR_FATAL;
       m_pStream->alignByte();
     }
     if (m_bIsGlobal) {
       std::unique_ptr<CJBig2_SymbolDict> value =
-          pSegment->m_Result.sd->DeepCopy();
+          pSegment->m_SymbolDict->DeepCopy();
       int size = pdfium::CollectionSize<int>(*m_pSymbolDictCache);
       while (size >= kSymbolDictCacheMaxSize) {
         m_pSymbolDictCache->pop_back();
@@ -630,9 +629,9 @@ int32_t CJBig2_Context::parseSymbolDict(CJBig2_Segment* pSegment,
   }
   if (wFlags & 0x0200) {
     if (bUseGbContext)
-      pSegment->m_Result.sd->SetGbContext(gbContext);
+      pSegment->m_SymbolDict->SetGbContext(gbContext);
     if (bUseGrContext)
-      pSegment->m_Result.sd->SetGrContext(grContext);
+      pSegment->m_SymbolDict->SetGrContext(grContext);
   }
   return JBIG2_SUCCESS;
 }
@@ -693,16 +692,16 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
     return JBIG2_ERROR_TOO_SHORT;
 
   for (int32_t i = 0; i < pSegment->m_nReferred_to_segment_count; ++i) {
-    if (!findSegmentByNumber(pSegment->m_pReferred_to_segment_numbers[i]))
+    if (!findSegmentByNumber(pSegment->m_Referred_to_segment_numbers[i]))
       return JBIG2_ERROR_FATAL;
   }
 
   pTRD->SBNUMSYMS = 0;
   for (int32_t i = 0; i < pSegment->m_nReferred_to_segment_count; ++i) {
     CJBig2_Segment* pSeg =
-        findSegmentByNumber(pSegment->m_pReferred_to_segment_numbers[i]);
+        findSegmentByNumber(pSegment->m_Referred_to_segment_numbers[i]);
     if (pSeg->m_cFlags.s.type == 0) {
-      pTRD->SBNUMSYMS += pSeg->m_Result.sd->NumImages();
+      pTRD->SBNUMSYMS += pSeg->m_SymbolDict->NumImages();
     }
   }
 
@@ -712,9 +711,9 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
     dwTemp = 0;
     for (int32_t i = 0; i < pSegment->m_nReferred_to_segment_count; ++i) {
       CJBig2_Segment* pSeg =
-          findSegmentByNumber(pSegment->m_pReferred_to_segment_numbers[i]);
+          findSegmentByNumber(pSegment->m_Referred_to_segment_numbers[i]);
       if (pSeg->m_cFlags.s.type == 0) {
-        const CJBig2_SymbolDict& dict = *pSeg->m_Result.sd;
+        const CJBig2_SymbolDict& dict = *pSeg->m_SymbolDict.get();
         for (size_t j = 0; j < dict.NumImages(); ++j)
           SBSYMS.get()[dwTemp + j] = dict.GetImage(j);
         dwTemp += dict.NumImages();
@@ -725,15 +724,14 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
     pTRD->SBSYMS = nullptr;
   }
 
-  std::unique_ptr<JBig2HuffmanCode, FxFreeDeleter> SBSYMCODES;
   if (pTRD->SBHUFF == 1) {
-    SBSYMCODES.reset(
-        decodeSymbolIDHuffmanTable(m_pStream.get(), pTRD->SBNUMSYMS));
-    if (!SBSYMCODES)
+    std::vector<JBig2HuffmanCode> SBSYMCODES =
+        decodeSymbolIDHuffmanTable(m_pStream.get(), pTRD->SBNUMSYMS);
+    if (SBSYMCODES.empty())
       return JBIG2_ERROR_FATAL;
 
     m_pStream->alignByte();
-    pTRD->SBSYMCODES = SBSYMCODES.get();
+    pTRD->SBSYMCODES = std::move(SBSYMCODES);
   } else {
     dwTemp = 0;
     while ((uint32_t)(1 << dwTemp) < pTRD->SBNUMSYMS) {
@@ -772,7 +770,7 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pTRD->SBHUFFFS = pSeg->m_Result.ht;
+      pTRD->SBHUFFFS = pSeg->m_HuffmanTable.get();
     }
     if (cSBHUFFDS == 0) {
       Table_B8 = pdfium::MakeUnique<CJBig2_HuffmanTable>(
@@ -791,7 +789,7 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pTRD->SBHUFFDS = pSeg->m_Result.ht;
+      pTRD->SBHUFFDS = pSeg->m_HuffmanTable.get();
     }
     if (cSBHUFFDT == 0) {
       Table_B11 = pdfium::MakeUnique<CJBig2_HuffmanTable>(
@@ -810,7 +808,7 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pTRD->SBHUFFDT = pSeg->m_Result.ht;
+      pTRD->SBHUFFDT = pSeg->m_HuffmanTable.get();
     }
     if (cSBHUFFRDW == 0) {
       Table_B14 = pdfium::MakeUnique<CJBig2_HuffmanTable>(
@@ -825,7 +823,7 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pTRD->SBHUFFRDW = pSeg->m_Result.ht;
+      pTRD->SBHUFFRDW = pSeg->m_HuffmanTable.get();
     }
     if (cSBHUFFRDH == 0) {
       if (!Table_B14) {
@@ -844,7 +842,7 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pTRD->SBHUFFRDH = pSeg->m_Result.ht;
+      pTRD->SBHUFFRDH = pSeg->m_HuffmanTable.get();
     }
     if (cSBHUFFRDX == 0) {
       if (!Table_B14) {
@@ -863,7 +861,7 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pTRD->SBHUFFRDX = pSeg->m_Result.ht;
+      pTRD->SBHUFFRDX = pSeg->m_HuffmanTable.get();
     }
     if (cSBHUFFRDY == 0) {
       if (!Table_B14) {
@@ -882,7 +880,7 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pTRD->SBHUFFRDY = pSeg->m_Result.ht;
+      pTRD->SBHUFFRDY = pSeg->m_HuffmanTable.get();
     }
     if (cSBHUFFRSIZE == 0) {
       Table_B1 = pdfium::MakeUnique<CJBig2_HuffmanTable>(
@@ -893,7 +891,7 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
           findReferredSegmentByTypeAndIndex(pSegment, 53, nIndex++);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
-      pTRD->SBHUFFRSIZE = pSeg->m_Result.ht;
+      pTRD->SBHUFFRSIZE = pSeg->m_HuffmanTable.get();
     }
   }
   std::unique_ptr<JBig2ArithCtx, FxFreeDeleter> grContext;
@@ -906,17 +904,16 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
     auto pArithDecoder =
         pdfium::MakeUnique<CJBig2_ArithDecoder>(m_pStream.get());
     pSegment->m_nResultType = JBIG2_IMAGE_POINTER;
-    pSegment->m_Result.im =
+    pSegment->m_Image =
         pTRD->decode_Arith(pArithDecoder.get(), grContext.get(), nullptr);
-    if (!pSegment->m_Result.im)
+    if (!pSegment->m_Image)
       return JBIG2_ERROR_FATAL;
     m_pStream->alignByte();
     m_pStream->offset(2);
   } else {
     pSegment->m_nResultType = JBIG2_IMAGE_POINTER;
-    pSegment->m_Result.im =
-        pTRD->decode_Huffman(m_pStream.get(), grContext.get());
-    if (!pSegment->m_Result.im)
+    pSegment->m_Image = pTRD->decode_Huffman(m_pStream.get(), grContext.get());
+    if (!pSegment->m_Image)
       return JBIG2_ERROR_FATAL;
     m_pStream->alignByte();
   }
@@ -928,16 +925,15 @@ int32_t CJBig2_Context::parseTextRegion(CJBig2_Segment* pSegment) {
         m_pPage->expand(ri.y + ri.height, (pPageInfo->m_cFlags & 4) ? 1 : 0);
       }
     }
-    m_pPage->composeFrom(ri.x, ri.y, pSegment->m_Result.im,
+    m_pPage->composeFrom(ri.x, ri.y, pSegment->m_Image.get(),
                          (JBig2ComposeOp)(ri.flags & 0x03));
-    delete pSegment->m_Result.im;
-    pSegment->m_Result.im = nullptr;
+    pSegment->m_Image.reset();
   }
   return JBIG2_SUCCESS;
 }
 
 int32_t CJBig2_Context::parsePatternDict(CJBig2_Segment* pSegment,
-                                         IFX_Pause* pPause) {
+                                         IFX_PauseIndicator* pPause) {
   uint8_t cFlags;
   auto pPDD = pdfium::MakeUnique<CJBig2_PDDProc>();
   if (m_pStream->read1Byte(&cFlags) != 0 ||
@@ -959,16 +955,16 @@ int32_t CJBig2_Context::parsePatternDict(CJBig2_Segment* pSegment,
     JBIG2_memset(gbContext.get(), 0, sizeof(JBig2ArithCtx) * size);
     auto pArithDecoder =
         pdfium::MakeUnique<CJBig2_ArithDecoder>(m_pStream.get());
-    pSegment->m_Result.pd =
+    pSegment->m_PatternDict =
         pPDD->decode_Arith(pArithDecoder.get(), gbContext.get(), pPause);
-    if (!pSegment->m_Result.pd)
+    if (!pSegment->m_PatternDict)
       return JBIG2_ERROR_FATAL;
 
     m_pStream->alignByte();
     m_pStream->offset(2);
   } else {
-    pSegment->m_Result.pd = pPDD->decode_MMR(m_pStream.get(), pPause);
-    if (!pSegment->m_Result.pd)
+    pSegment->m_PatternDict = pPDD->decode_MMR(m_pStream.get());
+    if (!pSegment->m_PatternDict)
       return JBIG2_ERROR_FATAL;
     m_pStream->alignByte();
   }
@@ -976,7 +972,7 @@ int32_t CJBig2_Context::parsePatternDict(CJBig2_Segment* pSegment,
 }
 
 int32_t CJBig2_Context::parseHalftoneRegion(CJBig2_Segment* pSegment,
-                                            IFX_Pause* pPause) {
+                                            IFX_PauseIndicator* pPause) {
   uint8_t cFlags;
   JBig2RegionInfo ri;
   auto pHRD = pdfium::MakeUnique<CJBig2_HTRDProc>();
@@ -1005,16 +1001,16 @@ int32_t CJBig2_Context::parseHalftoneRegion(CJBig2_Segment* pSegment,
     return JBIG2_ERROR_FATAL;
 
   CJBig2_Segment* pSeg =
-      findSegmentByNumber(pSegment->m_pReferred_to_segment_numbers[0]);
+      findSegmentByNumber(pSegment->m_Referred_to_segment_numbers[0]);
   if (!pSeg || (pSeg->m_cFlags.s.type != 16))
     return JBIG2_ERROR_FATAL;
 
-  CJBig2_PatternDict* pPatternDict = pSeg->m_Result.pd;
+  const CJBig2_PatternDict* pPatternDict = pSeg->m_PatternDict.get();
   if (!pPatternDict || (pPatternDict->NUMPATS == 0))
     return JBIG2_ERROR_FATAL;
 
   pHRD->HNUMPATS = pPatternDict->NUMPATS;
-  pHRD->HPATS = pPatternDict->HDPATS;
+  pHRD->HPATS = &pPatternDict->HDPATS;
   pHRD->HPW = pPatternDict->HDPATS[0]->width();
   pHRD->HPH = pPatternDict->HDPATS[0]->height();
   pSegment->m_nResultType = JBIG2_IMAGE_POINTER;
@@ -1025,16 +1021,16 @@ int32_t CJBig2_Context::parseHalftoneRegion(CJBig2_Segment* pSegment,
     JBIG2_memset(gbContext.get(), 0, sizeof(JBig2ArithCtx) * size);
     auto pArithDecoder =
         pdfium::MakeUnique<CJBig2_ArithDecoder>(m_pStream.get());
-    pSegment->m_Result.im =
+    pSegment->m_Image =
         pHRD->decode_Arith(pArithDecoder.get(), gbContext.get(), pPause);
-    if (!pSegment->m_Result.im)
+    if (!pSegment->m_Image)
       return JBIG2_ERROR_FATAL;
 
     m_pStream->alignByte();
     m_pStream->offset(2);
   } else {
-    pSegment->m_Result.im = pHRD->decode_MMR(m_pStream.get(), pPause);
-    if (!pSegment->m_Result.im)
+    pSegment->m_Image = pHRD->decode_MMR(m_pStream.get());
+    if (!pSegment->m_Image)
       return JBIG2_ERROR_FATAL;
     m_pStream->alignByte();
   }
@@ -1046,16 +1042,15 @@ int32_t CJBig2_Context::parseHalftoneRegion(CJBig2_Segment* pSegment,
         m_pPage->expand(ri.y + ri.height, (pPageInfo->m_cFlags & 4) ? 1 : 0);
       }
     }
-    m_pPage->composeFrom(ri.x, ri.y, pSegment->m_Result.im,
+    m_pPage->composeFrom(ri.x, ri.y, pSegment->m_Image.get(),
                          (JBig2ComposeOp)(ri.flags & 0x03));
-    delete pSegment->m_Result.im;
-    pSegment->m_Result.im = nullptr;
+    pSegment->m_Image.reset();
   }
   return JBIG2_SUCCESS;
 }
 
 int32_t CJBig2_Context::parseGenericRegion(CJBig2_Segment* pSegment,
-                                           IFX_Pause* pPause) {
+                                           IFX_PauseIndicator* pPause) {
   if (!m_pGRD) {
     auto pGRD = pdfium::MakeUnique<CJBig2_GRDProc>();
     uint8_t cFlags;
@@ -1065,7 +1060,6 @@ int32_t CJBig2_Context::parseGenericRegion(CJBig2_Segment* pSegment,
     }
     if (m_ri.height < 0 || m_ri.width < 0)
       return JBIG2_FAILED;
-
     pGRD->GBW = m_ri.width;
     pGRD->GBH = m_ri.height;
     pGRD->MMR = cFlags & 0x01;
@@ -1074,15 +1068,13 @@ int32_t CJBig2_Context::parseGenericRegion(CJBig2_Segment* pSegment,
     if (pGRD->MMR == 0) {
       if (pGRD->GBTEMPLATE == 0) {
         for (int32_t i = 0; i < 8; ++i) {
-          if (m_pStream->read1Byte((uint8_t*)&pGRD->GBAT[i]) != 0) {
+          if (m_pStream->read1Byte((uint8_t*)&pGRD->GBAT[i]) != 0)
             return JBIG2_ERROR_TOO_SHORT;
-          }
         }
       } else {
         for (int32_t i = 0; i < 2; ++i) {
-          if (m_pStream->read1Byte((uint8_t*)&pGRD->GBAT[i]) != 0) {
+          if (m_pStream->read1Byte((uint8_t*)&pGRD->GBAT[i]) != 0)
             return JBIG2_ERROR_TOO_SHORT;
-          }
         }
       }
     }
@@ -1091,18 +1083,16 @@ int32_t CJBig2_Context::parseGenericRegion(CJBig2_Segment* pSegment,
   }
   pSegment->m_nResultType = JBIG2_IMAGE_POINTER;
   if (m_pGRD->MMR == 0) {
-    if (m_gbContext.empty()) {
-      const size_t size = GetHuffContextSize(m_pGRD->GBTEMPLATE);
-      m_gbContext.resize(size);
-    }
+    if (m_gbContext.empty())
+      m_gbContext.resize(GetHuffContextSize(m_pGRD->GBTEMPLATE));
     if (!m_pArithDecoder) {
       m_pArithDecoder =
           pdfium::MakeUnique<CJBig2_ArithDecoder>(m_pStream.get());
-      m_ProcessingStatus = m_pGRD->Start_decode_Arith(&pSegment->m_Result.im,
-                                                      m_pArithDecoder.get(),
-                                                      &m_gbContext[0], pPause);
+      m_ProcessingStatus = m_pGRD->Start_decode_Arith(
+          &pSegment->m_Image, m_pArithDecoder.get(), &m_gbContext[0], pPause);
     } else {
-      m_ProcessingStatus = m_pGRD->Continue_decode(pPause);
+      m_ProcessingStatus =
+          m_pGRD->Continue_decode(pPause, m_pArithDecoder.get());
     }
     if (m_ProcessingStatus == FXCODEC_STATUS_DECODE_TOBECONTINUE) {
       if (pSegment->m_cFlags.s.type != 36) {
@@ -1116,24 +1106,23 @@ int32_t CJBig2_Context::parseGenericRegion(CJBig2_Segment* pSegment,
         }
         FX_RECT Rect = m_pGRD->GetReplaceRect();
         m_pPage->composeFrom(m_ri.x + Rect.left, m_ri.y + Rect.top,
-                             pSegment->m_Result.im,
+                             pSegment->m_Image.get(),
                              (JBig2ComposeOp)(m_ri.flags & 0x03), &Rect);
       }
       return JBIG2_SUCCESS;
-    } else {
-      m_pArithDecoder.reset();
-      m_gbContext.clear();
-      if (!pSegment->m_Result.im) {
-        m_ProcessingStatus = FXCODEC_STATUS_ERROR;
-        m_pGRD.reset();
-        return JBIG2_ERROR_FATAL;
-      }
-      m_pStream->alignByte();
-      m_pStream->offset(2);
     }
+    m_pArithDecoder.reset();
+    m_gbContext.clear();
+    if (!pSegment->m_Image) {
+      m_ProcessingStatus = FXCODEC_STATUS_ERROR;
+      m_pGRD.reset();
+      return JBIG2_ERROR_FATAL;
+    }
+    m_pStream->alignByte();
+    m_pStream->offset(2);
   } else {
-    m_pGRD->Start_decode_MMR(&pSegment->m_Result.im, m_pStream.get(), pPause);
-    if (!pSegment->m_Result.im) {
+    m_pGRD->Start_decode_MMR(&pSegment->m_Image, m_pStream.get());
+    if (!pSegment->m_Image) {
       m_pGRD.reset();
       return JBIG2_ERROR_FATAL;
     }
@@ -1150,10 +1139,9 @@ int32_t CJBig2_Context::parseGenericRegion(CJBig2_Segment* pSegment,
     }
     FX_RECT Rect = m_pGRD->GetReplaceRect();
     m_pPage->composeFrom(m_ri.x + Rect.left, m_ri.y + Rect.top,
-                         pSegment->m_Result.im,
+                         pSegment->m_Image.get(),
                          (JBig2ComposeOp)(m_ri.flags & 0x03), &Rect);
-    delete pSegment->m_Result.im;
-    pSegment->m_Result.im = nullptr;
+    pSegment->m_Image.reset();
   }
   m_pGRD.reset();
   return JBIG2_SUCCESS;
@@ -1181,7 +1169,7 @@ int32_t CJBig2_Context::parseGenericRefinementRegion(CJBig2_Segment* pSegment) {
   if (pSegment->m_nReferred_to_segment_count > 0) {
     int32_t i;
     for (i = 0; i < pSegment->m_nReferred_to_segment_count; ++i) {
-      pSeg = findSegmentByNumber(pSegment->m_pReferred_to_segment_numbers[0]);
+      pSeg = findSegmentByNumber(pSegment->m_Referred_to_segment_numbers[0]);
       if (!pSeg)
         return JBIG2_ERROR_FATAL;
 
@@ -1193,7 +1181,7 @@ int32_t CJBig2_Context::parseGenericRefinementRegion(CJBig2_Segment* pSegment) {
     if (i >= pSegment->m_nReferred_to_segment_count)
       return JBIG2_ERROR_FATAL;
 
-    pGRRD->GRREFERENCE = pSeg->m_Result.im;
+    pGRRD->GRREFERENCE = pSeg->m_Image.get();
   } else {
     pGRRD->GRREFERENCE = m_pPage.get();
   }
@@ -1205,8 +1193,8 @@ int32_t CJBig2_Context::parseGenericRefinementRegion(CJBig2_Segment* pSegment) {
   JBIG2_memset(grContext.get(), 0, sizeof(JBig2ArithCtx) * size);
   auto pArithDecoder = pdfium::MakeUnique<CJBig2_ArithDecoder>(m_pStream.get());
   pSegment->m_nResultType = JBIG2_IMAGE_POINTER;
-  pSegment->m_Result.im = pGRRD->decode(pArithDecoder.get(), grContext.get());
-  if (!pSegment->m_Result.im)
+  pSegment->m_Image = pGRRD->decode(pArithDecoder.get(), grContext.get());
+  if (!pSegment->m_Image)
     return JBIG2_ERROR_FATAL;
 
   m_pStream->alignByte();
@@ -1219,22 +1207,21 @@ int32_t CJBig2_Context::parseGenericRefinementRegion(CJBig2_Segment* pSegment) {
         m_pPage->expand(ri.y + ri.height, (pPageInfo->m_cFlags & 4) ? 1 : 0);
       }
     }
-    m_pPage->composeFrom(ri.x, ri.y, pSegment->m_Result.im,
+    m_pPage->composeFrom(ri.x, ri.y, pSegment->m_Image.get(),
                          (JBig2ComposeOp)(ri.flags & 0x03));
-    delete pSegment->m_Result.im;
-    pSegment->m_Result.im = nullptr;
+    pSegment->m_Image.reset();
   }
   return JBIG2_SUCCESS;
 }
 
 int32_t CJBig2_Context::parseTable(CJBig2_Segment* pSegment) {
   pSegment->m_nResultType = JBIG2_HUFFMAN_TABLE_POINTER;
-  pSegment->m_Result.ht = nullptr;
+  pSegment->m_HuffmanTable.reset();
   auto pHuff = pdfium::MakeUnique<CJBig2_HuffmanTable>(m_pStream.get());
   if (!pHuff->IsOK())
     return JBIG2_ERROR_FATAL;
 
-  pSegment->m_Result.ht = pHuff.release();
+  pSegment->m_HuffmanTable = std::move(pHuff);
   m_pStream->alignByte();
   return JBIG2_SUCCESS;
 }
@@ -1250,35 +1237,35 @@ int32_t CJBig2_Context::parseRegionInfo(JBig2RegionInfo* pRI) {
   return JBIG2_SUCCESS;
 }
 
-JBig2HuffmanCode* CJBig2_Context::decodeSymbolIDHuffmanTable(
+std::vector<JBig2HuffmanCode> CJBig2_Context::decodeSymbolIDHuffmanTable(
     CJBig2_BitStream* pStream,
     uint32_t SBNUMSYMS) {
   const size_t kRunCodesSize = 35;
-  int32_t runcodes[kRunCodesSize];
-  int32_t runcodes_len[kRunCodesSize];
+  JBig2HuffmanCode huffman_codes[kRunCodesSize];
   for (size_t i = 0; i < kRunCodesSize; ++i) {
-    if (pStream->readNBits(4, &runcodes_len[i]) != 0)
-      return nullptr;
+    if (pStream->readNBits(4, &huffman_codes[i].codelen) != 0)
+      return std::vector<JBig2HuffmanCode>();
   }
-  huffman_assign_code(runcodes, runcodes_len, kRunCodesSize);
+  huffman_assign_code(huffman_codes, kRunCodesSize);
 
-  std::unique_ptr<JBig2HuffmanCode, FxFreeDeleter> SBSYMCODES(
-      FX_Alloc(JBig2HuffmanCode, SBNUMSYMS));
+  std::vector<JBig2HuffmanCode> SBSYMCODES(SBNUMSYMS);
   int32_t run = 0;
   int32_t i = 0;
-  while (i < (int)SBNUMSYMS) {
+  while (i < static_cast<int>(SBNUMSYMS)) {
     size_t j;
     int32_t nVal = 0;
     int32_t nBits = 0;
     uint32_t nTemp;
     while (true) {
-      if (pStream->read1Bit(&nTemp) != 0)
-        return nullptr;
+      if (nVal > std::numeric_limits<int32_t>::max() / 2 ||
+          pStream->read1Bit(&nTemp) != 0) {
+        return std::vector<JBig2HuffmanCode>();
+      }
 
       nVal = (nVal << 1) | nTemp;
       ++nBits;
       for (j = 0; j < kRunCodesSize; ++j) {
-        if (nBits == runcodes_len[j] && nVal == runcodes[j])
+        if (nBits == huffman_codes[j].codelen && nVal == huffman_codes[j].code)
           break;
       }
       if (j < kRunCodesSize)
@@ -1286,110 +1273,58 @@ JBig2HuffmanCode* CJBig2_Context::decodeSymbolIDHuffmanTable(
     }
     int32_t runcode = static_cast<int32_t>(j);
     if (runcode < 32) {
-      SBSYMCODES.get()[i].codelen = runcode;
+      SBSYMCODES[i].codelen = runcode;
       run = 0;
     } else if (runcode == 32) {
       if (pStream->readNBits(2, &nTemp) != 0)
-        return nullptr;
+        return std::vector<JBig2HuffmanCode>();
       run = nTemp + 3;
     } else if (runcode == 33) {
       if (pStream->readNBits(3, &nTemp) != 0)
-        return nullptr;
+        return std::vector<JBig2HuffmanCode>();
       run = nTemp + 3;
     } else if (runcode == 34) {
       if (pStream->readNBits(7, &nTemp) != 0)
-        return nullptr;
+        return std::vector<JBig2HuffmanCode>();
       run = nTemp + 11;
     }
     if (run > 0) {
       if (i + run > (int)SBNUMSYMS)
-        return nullptr;
+        return std::vector<JBig2HuffmanCode>();
       for (int32_t k = 0; k < run; ++k) {
-        if (runcode == 32 && i > 0) {
-          SBSYMCODES.get()[i + k].codelen = SBSYMCODES.get()[i - 1].codelen;
-        } else {
-          SBSYMCODES.get()[i + k].codelen = 0;
-        }
+        if (runcode == 32 && i > 0)
+          SBSYMCODES[i + k].codelen = SBSYMCODES[i - 1].codelen;
+        else
+          SBSYMCODES[i + k].codelen = 0;
       }
       i += run;
     } else {
       ++i;
     }
   }
-  huffman_assign_code(SBSYMCODES.get(), SBNUMSYMS);
-  return SBSYMCODES.release();
-}
-
-void CJBig2_Context::huffman_assign_code(int* CODES, int* PREFLEN, int NTEMP) {
-  // TODO(thestig) CJBig2_HuffmanTable::parseFromCodedBuffer() has similar code.
-  int CURLEN, LENMAX, CURCODE, CURTEMP, i;
-  int* LENCOUNT;
-  int* FIRSTCODE;
-  LENMAX = 0;
-  for (i = 0; i < NTEMP; ++i) {
-    if (PREFLEN[i] > LENMAX) {
-      LENMAX = PREFLEN[i];
-    }
-  }
-  LENCOUNT = FX_Alloc(int, LENMAX + 1);
-  JBIG2_memset(LENCOUNT, 0, sizeof(int) * (LENMAX + 1));
-  FIRSTCODE = FX_Alloc(int, LENMAX + 1);
-  for (i = 0; i < NTEMP; ++i) {
-    ++LENCOUNT[PREFLEN[i]];
-  }
-  CURLEN = 1;
-  FIRSTCODE[0] = 0;
-  LENCOUNT[0] = 0;
-  while (CURLEN <= LENMAX) {
-    FIRSTCODE[CURLEN] = (FIRSTCODE[CURLEN - 1] + LENCOUNT[CURLEN - 1]) << 1;
-    CURCODE = FIRSTCODE[CURLEN];
-    CURTEMP = 0;
-    while (CURTEMP < NTEMP) {
-      if (PREFLEN[CURTEMP] == CURLEN) {
-        CODES[CURTEMP] = CURCODE;
-        CURCODE = CURCODE + 1;
-      }
-      CURTEMP = CURTEMP + 1;
-    }
-    CURLEN = CURLEN + 1;
-  }
-  FX_Free(LENCOUNT);
-  FX_Free(FIRSTCODE);
+  huffman_assign_code(SBSYMCODES.data(), SBNUMSYMS);
+  return SBSYMCODES;
 }
 
 void CJBig2_Context::huffman_assign_code(JBig2HuffmanCode* SBSYMCODES,
                                          int NTEMP) {
-  int CURLEN, LENMAX, CURCODE, CURTEMP, i;
-  int* LENCOUNT;
-  int* FIRSTCODE;
-  LENMAX = 0;
-  for (i = 0; i < NTEMP; ++i) {
-    if (SBSYMCODES[i].codelen > LENMAX) {
-      LENMAX = SBSYMCODES[i].codelen;
-    }
-  }
-  LENCOUNT = FX_Alloc(int, (LENMAX + 1));
-  JBIG2_memset(LENCOUNT, 0, sizeof(int) * (LENMAX + 1));
-  FIRSTCODE = FX_Alloc(int, (LENMAX + 1));
-  for (i = 0; i < NTEMP; ++i) {
+  // TODO(thestig) CJBig2_HuffmanTable::parseFromCodedBuffer() has similar code.
+  int LENMAX = 0;
+  for (int i = 0; i < NTEMP; ++i)
+    LENMAX = std::max(LENMAX, SBSYMCODES[i].codelen);
+  std::vector<int> LENCOUNT(LENMAX + 1);
+  std::vector<int> FIRSTCODE(LENMAX + 1);
+  for (int i = 0; i < NTEMP; ++i)
     ++LENCOUNT[SBSYMCODES[i].codelen];
-  }
-  CURLEN = 1;
-  FIRSTCODE[0] = 0;
   LENCOUNT[0] = 0;
-  while (CURLEN <= LENMAX) {
+  for (int CURLEN = 1; CURLEN <= LENMAX; ++CURLEN) {
     FIRSTCODE[CURLEN] = (FIRSTCODE[CURLEN - 1] + LENCOUNT[CURLEN - 1]) << 1;
-    CURCODE = FIRSTCODE[CURLEN];
-    CURTEMP = 0;
-    while (CURTEMP < NTEMP) {
+    int CURCODE = FIRSTCODE[CURLEN];
+    for (int CURTEMP = 0; CURTEMP < NTEMP; ++CURTEMP) {
       if (SBSYMCODES[CURTEMP].codelen == CURLEN) {
         SBSYMCODES[CURTEMP].code = CURCODE;
         CURCODE = CURCODE + 1;
       }
-      CURTEMP = CURTEMP + 1;
     }
-    CURLEN = CURLEN + 1;
   }
-  FX_Free(LENCOUNT);
-  FX_Free(FIRSTCODE);
 }
