@@ -6,7 +6,7 @@
 
 #include "core/fxcrt/xml/cxml_attritem.h"
 
-bool CXML_AttrItem::Matches(const CFX_ByteString& space,
-                            const CFX_ByteString& name) const {
+bool CXML_AttrItem::Matches(const ByteString& space,
+                            const ByteString& name) const {
   return (space.IsEmpty() || m_QSpaceName == space) && m_AttrName == name;
 }
