@@ -6,9 +6,14 @@
 
 #include <algorithm>
 #include <memory>
+#include <sstream>
+#include <string>
+#include <utility>
 #include <vector>
 
-#include "core/fxcrt/fx_ext.h"
+#include "core/fxcrt/cfx_utf8decoder.h"
+#include "core/fxcrt/cfx_widetextbuf.h"
+#include "core/fxcrt/fx_extension.h"
 #include "core/fxcrt/xml/cxml_content.h"
 #include "core/fxcrt/xml/cxml_element.h"
 #include "core/fxcrt/xml/cxml_parser.h"
@@ -53,6 +58,8 @@ const uint8_t g_FXCRT_XML_ByteTypes[256] = {
     0x1A, 0x1A, 0x01, 0x01,
 };
 
+constexpr int kMaxDepth = 1024;
+
 bool g_FXCRT_XML_IsWhiteSpace(uint8_t ch) {
   return !!(g_FXCRT_XML_ByteTypes[ch] & FXCRTM_XML_CHARTYPE_SpaceChar);
 }
@@ -69,161 +76,20 @@ bool g_FXCRT_XML_IsNameChar(uint8_t ch) {
   return !!(g_FXCRT_XML_ByteTypes[ch] & FXCRTM_XML_CHARTYPE_NameChar);
 }
 
-class CXML_DataBufAcc : public IFX_BufferedReadStream {
- public:
-  template <typename T, typename... Args>
-  friend CFX_RetainPtr<T> pdfium::MakeRetain(Args&&... args);
-
-  // IFX_BufferedReadStream
-  bool IsEOF() override;
-  FX_FILESIZE GetPosition() override;
-  size_t ReadBlock(void* buffer, size_t size) override;
-  bool ReadNextBlock(bool bRestart) override;
-  const uint8_t* GetBlockBuffer() override;
-  size_t GetBlockSize() override;
-  FX_FILESIZE GetBlockOffset() override;
-
- private:
-  CXML_DataBufAcc(const uint8_t* pBuffer, size_t size);
-  ~CXML_DataBufAcc() override;
-
-  const uint8_t* m_pBuffer;
-  size_t m_dwSize;
-  size_t m_dwCurPos;
-};
-
-CXML_DataBufAcc::CXML_DataBufAcc(const uint8_t* pBuffer, size_t size)
-    : m_pBuffer(pBuffer), m_dwSize(size), m_dwCurPos(0) {}
-
-CXML_DataBufAcc::~CXML_DataBufAcc() {}
-
-bool CXML_DataBufAcc::IsEOF() {
-  return m_dwCurPos >= m_dwSize;
-}
-
-FX_FILESIZE CXML_DataBufAcc::GetPosition() {
-  return static_cast<FX_FILESIZE>(m_dwCurPos);
-}
-
-size_t CXML_DataBufAcc::ReadBlock(void* buffer, size_t size) {
-  return 0;
-}
-
-bool CXML_DataBufAcc::ReadNextBlock(bool bRestart) {
-  if (bRestart)
-    m_dwCurPos = 0;
-
-  if (m_dwCurPos < m_dwSize) {
-    m_dwCurPos = m_dwSize;
-    return true;
-  }
-  return false;
-}
-
-const uint8_t* CXML_DataBufAcc::GetBlockBuffer() {
-  return m_pBuffer;
-}
-
-size_t CXML_DataBufAcc::GetBlockSize() {
-  return m_dwSize;
-}
-
-FX_FILESIZE CXML_DataBufAcc::GetBlockOffset() {
-  return 0;
-}
-
-class CXML_DataStmAcc : public IFX_BufferedReadStream {
- public:
-  template <typename T, typename... Args>
-  friend CFX_RetainPtr<T> pdfium::MakeRetain(Args&&... args);
-
-  // IFX_BufferedReadStream
-  bool IsEOF() override;
-  FX_FILESIZE GetPosition() override;
-  size_t ReadBlock(void* buffer, size_t size) override;
-  bool ReadNextBlock(bool bRestart) override;
-  const uint8_t* GetBlockBuffer() override;
-  size_t GetBlockSize() override;
-  FX_FILESIZE GetBlockOffset() override;
-
- private:
-  explicit CXML_DataStmAcc(
-      const CFX_RetainPtr<IFX_SeekableReadStream>& pFileRead);
-  ~CXML_DataStmAcc() override;
-
-  CFX_RetainPtr<IFX_SeekableReadStream> m_pFileRead;
-  uint8_t* m_pBuffer;
-  FX_FILESIZE m_nStart;
-  size_t m_dwSize;
-};
-
-CXML_DataStmAcc::CXML_DataStmAcc(
-    const CFX_RetainPtr<IFX_SeekableReadStream>& pFileRead)
-    : m_pFileRead(pFileRead), m_pBuffer(nullptr), m_nStart(0), m_dwSize(0) {
-  ASSERT(m_pFileRead);
-}
-
-CXML_DataStmAcc::~CXML_DataStmAcc() {
-  FX_Free(m_pBuffer);
-}
-
-bool CXML_DataStmAcc::IsEOF() {
-  return m_nStart + static_cast<FX_FILESIZE>(m_dwSize) >=
-         m_pFileRead->GetSize();
-}
-
-FX_FILESIZE CXML_DataStmAcc::GetPosition() {
-  return m_nStart + static_cast<FX_FILESIZE>(m_dwSize);
-}
-
-size_t CXML_DataStmAcc::ReadBlock(void* buffer, size_t size) {
-  return 0;
-}
-
-bool CXML_DataStmAcc::ReadNextBlock(bool bRestart) {
-  if (bRestart)
-    m_nStart = 0;
-
-  FX_FILESIZE nLength = m_pFileRead->GetSize();
-  m_nStart += static_cast<FX_FILESIZE>(m_dwSize);
-  if (m_nStart >= nLength)
-    return false;
-
-  static const FX_FILESIZE FX_XMLDATASTREAM_BufferSize = 32 * 1024;
-  m_dwSize = static_cast<size_t>(
-      std::min(FX_XMLDATASTREAM_BufferSize, nLength - m_nStart));
-  if (!m_pBuffer)
-    m_pBuffer = FX_Alloc(uint8_t, m_dwSize);
-
-  return m_pFileRead->ReadBlock(m_pBuffer, m_nStart, m_dwSize);
-}
-
-const uint8_t* CXML_DataStmAcc::GetBlockBuffer() {
-  return (const uint8_t*)m_pBuffer;
-}
-
-size_t CXML_DataStmAcc::GetBlockSize() {
-  return m_dwSize;
-}
-
-FX_FILESIZE CXML_DataStmAcc::GetBlockOffset() {
-  return m_nStart;
-}
-
 }  // namespace
 
-void FX_XML_SplitQualifiedName(const CFX_ByteStringC& bsFullName,
-                               CFX_ByteStringC& bsSpace,
-                               CFX_ByteStringC& bsName) {
+void FX_XML_SplitQualifiedName(const ByteStringView& bsFullName,
+                               ByteStringView& bsSpace,
+                               ByteStringView& bsName) {
   if (bsFullName.IsEmpty())
     return;
 
-  FX_STRSIZE iStart = bsFullName.Find(':');
-  if (iStart == -1) {
+  auto iStart = bsFullName.Find(':');
+  if (!iStart.has_value()) {
     bsName = bsFullName;
   } else {
-    bsSpace = bsFullName.Mid(0, iStart);
-    bsName = bsFullName.Mid(iStart + 1);
+    bsSpace = bsFullName.Left(iStart.value());
+    bsName = bsFullName.Right(bsFullName.GetLength() - (iStart.value() + 1));
   }
 }
 
@@ -237,7 +103,7 @@ CXML_Parser::CXML_Parser()
 CXML_Parser::~CXML_Parser() {}
 
 bool CXML_Parser::Init(const uint8_t* pBuffer, size_t size) {
-  m_pDataAcc = pdfium::MakeRetain<CXML_DataBufAcc>(pBuffer, size);
+  m_pDataAcc = pdfium::MakeUnique<CXML_DataBufAcc>(pBuffer, size);
   m_nOffset = 0;
   return ReadNextBlock();
 }
@@ -248,7 +114,7 @@ bool CXML_Parser::ReadNextBlock() {
 
   m_pBuffer = m_pDataAcc->GetBlockBuffer();
   m_dwBufferSize = m_pDataAcc->GetBlockSize();
-  m_nBufferOffset = m_pDataAcc->GetBlockOffset();
+  m_nBufferOffset = 0;
   m_dwIndex = 0;
   return m_dwBufferSize > 0;
 }
@@ -273,21 +139,20 @@ void CXML_Parser::SkipWhiteSpaces() {
   } while (ReadNextBlock());
 }
 
-void CXML_Parser::GetName(CFX_ByteString* space, CFX_ByteString* name) {
+void CXML_Parser::GetName(ByteString* space, ByteString* name) {
   m_nOffset = m_nBufferOffset + static_cast<FX_FILESIZE>(m_dwIndex);
   if (IsEOF())
     return;
 
-  CFX_ByteTextBuf buf;
-  uint8_t ch;
+  std::ostringstream buf;
   do {
     while (m_dwIndex < m_dwBufferSize) {
-      ch = m_pBuffer[m_dwIndex];
+      uint8_t ch = m_pBuffer[m_dwIndex];
       if (ch == ':') {
-        *space = buf.AsStringC();
-        buf.Clear();
+        *space = ByteString(buf);
+        buf.str("");
       } else if (g_FXCRT_XML_IsNameChar(ch)) {
-        buf.AppendChar(ch);
+        buf << static_cast<char>(ch);
       } else {
         break;
       }
@@ -297,10 +162,10 @@ void CXML_Parser::GetName(CFX_ByteString* space, CFX_ByteString* name) {
     if (m_dwIndex < m_dwBufferSize || IsEOF())
       break;
   } while (ReadNextBlock());
-  *name = buf.AsStringC();
+  *name = ByteString(buf);
 }
 
-void CXML_Parser::SkipLiterals(const CFX_ByteStringC& str) {
+void CXML_Parser::SkipLiterals(const ByteStringView& str) {
   m_nOffset = m_nBufferOffset + static_cast<FX_FILESIZE>(m_dwIndex);
   if (IsEOF()) {
     return;
@@ -308,7 +173,7 @@ void CXML_Parser::SkipLiterals(const CFX_ByteStringC& str) {
   int32_t i = 0, iLen = str.GetLength();
   do {
     while (m_dwIndex < m_dwBufferSize) {
-      if (str.GetAt(i) != m_pBuffer[m_dwIndex++]) {
+      if (str[i] != m_pBuffer[m_dwIndex++]) {
         i = 0;
         continue;
       }
@@ -337,7 +202,7 @@ uint32_t CXML_Parser::GetCharRef() {
 
   uint8_t ch;
   int32_t iState = 0;
-  CFX_ByteTextBuf buf;
+  std::ostringstream buf;
   uint32_t code = 0;
   do {
     while (m_dwIndex < m_dwBufferSize) {
@@ -353,7 +218,7 @@ uint32_t CXML_Parser::GetCharRef() {
         case 1:
           m_dwIndex++;
           if (ch == ';') {
-            CFX_ByteStringC ref = buf.AsStringC();
+            std::string ref = buf.str();
             if (ref == "gt")
               code = '>';
             else if (ref == "lt")
@@ -367,7 +232,7 @@ uint32_t CXML_Parser::GetCharRef() {
             iState = 10;
             break;
           }
-          buf.AppendByte(ch);
+          buf << static_cast<char>(ch);
           break;
         case 2:
           if (ch == 'x') {
@@ -383,7 +248,7 @@ uint32_t CXML_Parser::GetCharRef() {
             break;
           }
           if (g_FXCRT_XML_IsDigital(ch))
-            code = code * 10 + FXSYS_toDecimalDigit(static_cast<wchar_t>(ch));
+            code = code * 10 + FXSYS_DecimalCharToInt(static_cast<wchar_t>(ch));
           break;
         case 4:
           m_dwIndex++;
@@ -395,8 +260,8 @@ uint32_t CXML_Parser::GetCharRef() {
               g_FXCRT_XML_ByteTypes[ch] & FXCRTM_XML_CHARTYPE_HexChar;
           if (nHex) {
             if (nHex == FXCRTM_XML_CHARTYPE_HexDigital) {
-              code =
-                  (code << 4) + FXSYS_toDecimalDigit(static_cast<wchar_t>(ch));
+              code = (code << 4) +
+                     FXSYS_DecimalCharToInt(static_cast<wchar_t>(ch));
             } else if (nHex == FXCRTM_XML_CHARTYPE_HexLowerLetter) {
               code = (code << 4) + ch - 87;
             } else {
@@ -416,7 +281,7 @@ uint32_t CXML_Parser::GetCharRef() {
   return code;
 }
 
-void CXML_Parser::GetAttrValue(CFX_WideString& value) {
+void CXML_Parser::GetAttrValue(WideString& value) {
   m_nOffset = m_nBufferOffset + static_cast<FX_FILESIZE>(m_dwIndex);
   if (IsEOF())
     return;
@@ -440,7 +305,7 @@ void CXML_Parser::GetAttrValue(CFX_WideString& value) {
         break;
 
       if (ch == '&') {
-        decoder.AppendChar(GetCharRef());
+        decoder.AppendCodePoint(GetCharRef());
         if (IsEOF()) {
           value = decoder.GetResult();
           return;
@@ -458,8 +323,8 @@ void CXML_Parser::GetAttrValue(CFX_WideString& value) {
 
 void CXML_Parser::GetTagName(bool bStartTag,
                              bool* bEndTag,
-                             CFX_ByteString* space,
-                             CFX_ByteString* name) {
+                             ByteString* space,
+                             ByteString* name) {
   m_nOffset = m_nBufferOffset + static_cast<FX_FILESIZE>(m_dwIndex);
   if (IsEOF())
     return;
@@ -510,22 +375,32 @@ void CXML_Parser::GetTagName(bool bStartTag,
 
 std::unique_ptr<CXML_Element> CXML_Parser::ParseElement(CXML_Element* pParent,
                                                         bool bStartTag) {
+  return ParseElementInternal(pParent, bStartTag, 0);
+}
+
+std::unique_ptr<CXML_Element> CXML_Parser::ParseElementInternal(
+    CXML_Element* pParent,
+    bool bStartTag,
+    int nDepth) {
+  if (nDepth > kMaxDepth)
+    return nullptr;
+
   m_nOffset = m_nBufferOffset + static_cast<FX_FILESIZE>(m_dwIndex);
   if (IsEOF())
     return nullptr;
 
-  CFX_ByteString tag_name;
-  CFX_ByteString tag_space;
+  ByteString tag_name;
+  ByteString tag_space;
   bool bEndTag;
   GetTagName(bStartTag, &bEndTag, &tag_space, &tag_name);
   if (tag_name.IsEmpty() || bEndTag)
     return nullptr;
 
   auto pElement = pdfium::MakeUnique<CXML_Element>(
-      pParent, tag_space.AsStringC(), tag_name.AsStringC());
+      pParent, tag_space.AsStringView(), tag_name.AsStringView());
   do {
-    CFX_ByteString attr_space;
-    CFX_ByteString attr_name;
+    ByteString attr_space;
+    ByteString attr_name;
     while (m_dwIndex < m_dwBufferSize) {
       SkipWhiteSpaces();
       if (IsEOF())
@@ -547,9 +422,9 @@ std::unique_ptr<CXML_Element> CXML_Parser::ParseElement(CXML_Element* pParent,
       if (IsEOF())
         break;
 
-      CFX_WideString attr_value;
+      WideString attr_value;
       GetAttrValue(attr_value);
-      pElement->m_AttrMap.SetAt(attr_space, attr_name, attr_value);
+      pElement->SetAttribute(attr_space, attr_name, attr_value);
     }
     m_nOffset = m_nBufferOffset + static_cast<FX_FILESIZE>(m_dwIndex);
     if (m_dwIndex < m_dwBufferSize || IsEOF())
@@ -586,7 +461,7 @@ std::unique_ptr<CXML_Element> CXML_Parser::ParseElement(CXML_Element* pParent,
             iState = 1;
           } else if (ch == '&') {
             decoder.ClearStatus();
-            decoder.AppendChar(GetCharRef());
+            decoder.AppendCodePoint(GetCharRef());
           } else {
             decoder.Input(ch);
           }
@@ -599,31 +474,31 @@ std::unique_ptr<CXML_Element> CXML_Parser::ParseElement(CXML_Element* pParent,
             SkipWhiteSpaces();
             iState = 0;
           } else if (ch == '/') {
-            CFX_ByteString space;
-            CFX_ByteString name;
+            ByteString space;
+            ByteString name;
             GetName(&space, &name);
             SkipWhiteSpaces();
             m_dwIndex++;
             iState = 10;
           } else {
             content << decoder.GetResult();
-            CFX_WideString dataStr = content.MakeString();
+            WideString dataStr = content.MakeString();
             if (!bCDATA)
               dataStr.TrimRight(L" \t\r\n");
 
-            InsertContentSegment(bCDATA, dataStr.AsStringC(), pElement.get());
+            InsertContentSegment(bCDATA, dataStr.AsStringView(),
+                                 pElement.get());
             content.Clear();
             decoder.Clear();
             bCDATA = false;
             iState = 0;
             m_dwIndex--;
-            std::unique_ptr<CXML_Element> pSubElement(
-                ParseElement(pElement.get(), true));
+            std::unique_ptr<CXML_Element> pSubElement =
+                ParseElementInternal(pElement.get(), true, nDepth + 1);
             if (!pSubElement)
               break;
 
-            pElement->m_Children.push_back(
-                {CXML_Element::Element, pSubElement.release()});
+            pElement->AppendChild(std::move(pSubElement));
             SkipWhiteSpaces();
           }
           break;
@@ -650,10 +525,10 @@ std::unique_ptr<CXML_Element> CXML_Parser::ParseElement(CXML_Element* pParent,
       break;
   } while (ReadNextBlock());
   content << decoder.GetResult();
-  CFX_WideString dataStr = content.MakeString();
+  WideString dataStr = content.MakeString();
   dataStr.TrimRight(L" \t\r\n");
 
-  InsertContentSegment(bCDATA, dataStr.AsStringC(), pElement.get());
+  InsertContentSegment(bCDATA, dataStr.AsStringView(), pElement.get());
   content.Clear();
   decoder.Clear();
   bCDATA = false;
@@ -661,20 +536,10 @@ std::unique_ptr<CXML_Element> CXML_Parser::ParseElement(CXML_Element* pParent,
 }
 
 void CXML_Parser::InsertContentSegment(bool bCDATA,
-                                       const CFX_WideStringC& content,
+                                       const WideStringView& content,
                                        CXML_Element* pElement) {
   if (content.IsEmpty())
     return;
 
-  CXML_Content* pContent = new CXML_Content;
-  pContent->Set(bCDATA, content);
-  pElement->m_Children.push_back({CXML_Element::Content, pContent});
-}
-
-std::unique_ptr<CXML_Element> CXML_Element::Parse(const void* pBuffer,
-                                                  size_t size) {
-  CXML_Parser parser;
-  if (!parser.Init(static_cast<const uint8_t*>(pBuffer), size))
-    return nullptr;
-  return parser.ParseElement(nullptr, false);
+  pElement->AppendChild(pdfium::MakeUnique<CXML_Content>(bCDATA, content));
 }
