@@ -257,14 +257,14 @@ std::ostream &operator<<(std::ostream &o, WPSGraphicShape const &sh)
 		break;
 	case WPSGraphicShape::Polygon:
 		o << "polygons,pts=[";
-		for (size_t pt=0; pt < sh.m_vertices.size(); ++pt)
-			o << sh.m_vertices[pt] << ",";
+		for (auto const &pt : sh.m_vertices)
+			o << pt << ",";
 		o << "],";
 		break;
 	case WPSGraphicShape::Path:
 		o << "path,pts=[";
-		for (size_t pt=0; pt < sh.m_path.size(); ++pt)
-			o << sh.m_path[pt] << ",";
+		for (auto const &pt : sh.m_path)
+			o << pt << ",";
 		o << "],";
 		break;
 	case WPSGraphicShape::ShapeUnknown:
@@ -311,10 +311,10 @@ void WPSGraphicShape::translate(Vec2f const &decal)
 		return;
 	m_bdBox=WPSBox2f(m_bdBox.min()+decal, m_bdBox.max()+decal);
 	m_formBox=WPSBox2f(m_formBox.min()+decal, m_formBox.max()+decal);
-	for (size_t pt=0; pt<m_vertices.size(); ++pt)
-		m_vertices[pt]+=decal;
-	for (size_t pt=0; pt<m_path.size(); ++pt)
-		m_path[pt].translate(decal);
+	for (auto &pt : m_vertices)
+		pt+=decal;
+	for (auto &pt : m_path)
+		pt.translate(decal);
 }
 
 void WPSGraphicShape::scale(Vec2f const &scaling)
@@ -323,11 +323,11 @@ void WPSGraphicShape::scale(Vec2f const &scaling)
 	                 Vec2f(scaling[0]*m_bdBox.max()[0],scaling[1]*m_bdBox.max()[1]));
 	m_formBox=WPSBox2f(Vec2f(scaling[0]*m_formBox.min()[0],scaling[1]*m_formBox.min()[1]),
 	                   Vec2f(scaling[0]*m_formBox.max()[0],scaling[1]*m_formBox.max()[1]));
-	for (size_t pt=0; pt<m_vertices.size(); ++pt)
-		m_vertices[pt]=Vec2f(scaling[0]*m_vertices[pt][0],
-		                     scaling[1]*m_vertices[pt][1]);
-	for (size_t pt=0; pt<m_path.size(); ++pt)
-		m_path[pt].scale(scaling);
+	for (auto &vertex : m_vertices)
+		vertex=Vec2f(scaling[0]*vertex[0],
+		             scaling[1]*vertex[1]);
+	for (auto &pt : m_path)
+		pt.scale(scaling);
 }
 
 WPSGraphicShape WPSGraphicShape::rotate(float angle, Vec2f const &center) const
@@ -349,8 +349,8 @@ WPSGraphicShape WPSGraphicShape::rotate(float angle, Vec2f const &center) const
 	}
 	WPSGraphicShape res = path(fBox);
 	res.m_path=getPath(false);
-	for (size_t p=0; p < res.m_path.size(); p++)
-		res.m_path[p].rotate(angle, decal);
+	for (auto &pt : res.m_path)
+		pt.rotate(angle, decal);
 	return res;
 }
 
@@ -381,8 +381,8 @@ WPSGraphicShape WPSGraphicShape::transform(WPSTransformation const &matrix) cons
 	Vec2f shearing;
 	if (!matrix.decompose(rotation,shearing,transf,fBox.center()))
 		rotation=0;
-	for (size_t p=0; p < res.m_path.size(); p++)
-		res.m_path[p].transform(matrix, rotation);
+	for (auto &p : res.m_path)
+		p.transform(matrix, rotation);
 	return res;
 }
 
@@ -631,6 +631,7 @@ std::vector<WPSGraphicShape::PathData> WPSGraphicShape::getPath(bool forTransfor
 			break;
 		}
 	}
+	WPS_FALLTHROUGH;
 	case Arc:
 	case Pie:
 	{
