@@ -12,6 +12,10 @@
 #include "xfa/fwl/ifwl_adaptertimermgr.h"
 #include "xfa/fxfa/cxfa_ffapp.h"
 
+CFWL_Timer::CFWL_Timer(CFWL_Widget* parent) : m_pWidget(parent) {}
+
+CFWL_Timer::~CFWL_Timer() {}
+
 CFWL_TimerInfo* CFWL_Timer::StartTimer(uint32_t dwElapse, bool bImmediately) {
   const CFWL_App* pApp = m_pWidget->GetOwnerApp();
   if (!pApp)
