@@ -18,7 +18,7 @@ class CFWL_ListBoxTP : public CFWL_WidgetTP {
   void DrawBackground(CFWL_ThemeBackground* pParams) override;
 
  protected:
-  void DrawListBoxItem(CFX_Graphics* pGraphics,
+  void DrawListBoxItem(CXFA_Graphics* pGraphics,
                        uint32_t dwStates,
                        const CFX_RectF* prtItem,
                        void* pData = nullptr,
